import colorsys
import os
import sys
import numpy as np
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *
import pyqtgraph as pg
from pyqtgraph.Qt import QtGui, QtCore
import pyqtgraph.functions as fn
from .movable_points import TriangulationPoints, TriangulationPointsTest


class ClickableImage(pg.ImageItem):
    class SignalProxy(QObject):
        mouseHovered = pyqtSignal(object)  # id
        mouseClicked = pyqtSignal(object)  # id

    def __init__(self):
        self._sigprox = ClickableImage.SignalProxy()
        self.mouseHovered = self._sigprox.mouseHovered
        self.mouseClicked = self._sigprox.mouseClicked

        pg.ImageItem.__init__(self)

        self.setAcceptHoverEvents(True)
        # self.setOpts(axisOrder='row-major')

    def set_data(self, hist_image_data, scale=None):
        # self.label_data = label
        self.hist_data = hist_image_data
        if scale is not None:
            self.resetTransform()
            self.scale(*scale)
        self.setImage(self.hist_data, autoLevels=False)
        # self.label_img.setImage(self.label_data, autoLevels=False)

    def hoverEvent(self, event):
        if event.isExit():
            return
        try:
            pos = (event.pos())
            id = 1  # self.label_data[int(event.pos().x()), int(event.pos().y())]
        except (IndexError, AttributeError):
            return
        self.mouseHovered.emit(event)

    def mouseClickEvent(self, event):
        pos = (event.pos().x(), event.pos().y())
        # print(pos)
        # id = self.label_data[int(event.pos().x()), int(event.pos().y())]
        self.mouseClicked.emit(pos)
        # self.mouseClicked.emit([event, id])


class SliceStack(pg.GraphicsLayoutWidget):
    class SignalProxy(QObject):
        mouseHovered = pyqtSignal(object)  # id
        mouseClicked = pyqtSignal(object)  # id
        sigKeyPressed = pyqtSignal(object)  # event

    def __init__(self):
        self._sigprox = SliceStack.SignalProxy()
        self.sig_mouse_hovered = self._sigprox.mouseHovered
        self.sig_mouse_clicked = self._sigprox.mouseClicked
        self.sig_key_pressed = self._sigprox.sigKeyPressed

        pg.GraphicsLayoutWidget.__init__(self)

        self.data = None
        self.vb = self.addViewBox()
        self.setBackground('k')
        self.vb.setAspectLocked()
        self.vb.invertY(True)
        self.vb.setAcceptHoverEvents(True)
        # self.vb.enableAutoRange(enable=False)

        self.base_layer = ClickableImage()
        self.base_layer.mouseClicked.connect(self.mouse_clicked)
        self.base_layer.mouseHovered.connect(self.mouse_hovered)

        self.img_layer = pg.ImageItem()
        self.img_layer.setCompositionMode(QPainter.CompositionMode_Plus)

        circle_follow = pg.PlotDataItem(pen=pg.mkPen('r', width=2, style=Qt.DashLine))
        lasso_path = pg.PlotDataItem(pen=pg.mkPen(color=(0, 255, 255), width=3, style=Qt.DashLine),
                                     symbolPen=(0, 255, 255), symbol='o', symbolSize=4)

        ruler_path = pg.PlotDataItem(pen=pg.mkPen(color='y', width=3, style=Qt.DashLine),
                                     symbolPen='y', symbolBrush='y', symbol='s', symbolSize=4)

        overlay_img = pg.ImageItem()
        overlay_img.setCompositionMode(QPainter.CompositionMode_Plus)
        overlay_contour = pg.ImageItem()
        overlay_contour.setLevels(levels=(0, 1))

        mask_img = pg.ImageItem()
        mask_img.setLevels(levels=(0, 1))

        tri_pnts = TriangulationPoints()
        tri_pnts.setVisible(False)
        self.tri_lines_list = []

        grid_lines = pg.GridItem(pen=(128, 128, 128))
        grid_lines.setVisible(False)

        cell_pnts = pg.ScatterPlotItem(pen=(0, 255, 0), brush=(0, 255, 0), size=5, hoverSize=8)
        probe_pnts = pg.ScatterPlotItem(pen=(0, 0, 255), brush=(0, 0, 255), symbol='s', size=5, hoverSize=8)
        bregma_pnt = pg.ScatterPlotItem(pen=(255, 255, 0), brush=(255, 255, 0), symbol='star', size=10, hoverSize=15)

        drawing_pnts = pg.PlotDataItem(pen=pg.mkPen(color=(255, 102, 0), width=2), brush=None)
        contour_pnts = pg.PlotDataItem(pen=pg.mkPen(color=(255, 0, 255), width=3), brush=None)

        self.pre_trajectory_list = []
        for i in range(4):
            self.pre_trajectory_list.append(pg.PlotDataItem(pen=pg.mkPen(color=(0, 0, 255), width=2), brush=None,
                                                            symbolPen=(0, 0, 255), symbolBrush=(0, 0, 255),
                                                            symbol='s', symbolSize=3))

        self.image_dict = {'atlas-overlay': overlay_img,
                           'overlay_contour': overlay_contour,
                           'atlas-drawing': drawing_pnts,
                           'atlas-contour': contour_pnts,
                           'atlas-mask': mask_img,
                           'grid_lines': grid_lines,
                           'tri_pnts': tri_pnts,
                           'circle_follow': circle_follow,
                           'lasso_path': lasso_path,
                           'atlas-probe': probe_pnts,
                           'bregma_pnt': bregma_pnt,
                           'ruler_path': ruler_path,
                           'atlas-cells': cell_pnts}
        self.image_dict_keys = list(self.image_dict.keys())

        self.vb.addItem(self.base_layer)
        self.vb.addItem(self.img_layer)

        for i in range(len(self.image_dict)):
            self.vb.addItem(self.image_dict[self.image_dict_keys[i]])

        for i in range(4):
            self.vb.addItem(self.pre_trajectory_list[i])

    def set_data(self, data, scale=None):
        self.data = data
        if scale is not None:
            self.resetTransform()
            self.scale(*scale)
        base_img = np.zeros(data.shape[:2], 'uint8')
        self.base_layer.setImage(base_img)
        self.img_layer.setImage(data)

    def boundingRect(self):
        return self.base_layer.boundingRect()

    def mouse_hovered(self, event):
        if event.isExit():
            return
        try:
            pos = (event.pos())
            id = 1  # self.label_data[int(event.pos().x()), int(event.pos().y())]
        except (IndexError, AttributeError):
            return
        self.sig_mouse_hovered.emit(event)

    def mouse_clicked(self, pos):
        self.sig_mouse_clicked.emit(pos)

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Backspace:
            print("Killing")
            self.sig_key_pressed.emit('delete')


class ImageStacks(pg.GraphicsLayoutWidget):
    class SignalProxy(QObject):
        sigMouseHovered = pyqtSignal(object)  # id
        sigMouseClicked = pyqtSignal(object)  # id
        sigKeyPressed = pyqtSignal(object)  # event

    def __init__(self):
        self._sigprox = ImageStacks.SignalProxy()
        self.sig_mouse_hovered = self._sigprox.sigMouseHovered
        self.sig_mouse_clicked = self._sigprox.sigMouseClicked
        self.sig_key_pressed = self._sigprox.sigKeyPressed

        pg.GraphicsLayoutWidget.__init__(self)

        self.data = None
        self.vb = self.addViewBox()
        self.setBackground('k')
        self.vb.setAspectLocked()
        self.vb.invertY(True)
        self.vb.setAcceptHoverEvents(True)
        # self.vb.enableAutoRange(enable=False)

        self.base_layer = ClickableImage()
        self.base_layer.mouseClicked.connect(self.mouse_clicked)
        self.base_layer.mouseHovered.connect(self.mouse_hovered)

        img1 = pg.ImageItem()
        img1.setCompositionMode(QPainter.CompositionMode_Plus)
        img2 = pg.ImageItem()
        img2.setCompositionMode(QPainter.CompositionMode_Plus)
        img3 = pg.ImageItem()
        img3.setCompositionMode(QPainter.CompositionMode_Plus)
        img4 = pg.ImageItem()
        img4.setCompositionMode(QPainter.CompositionMode_Plus)

        circle_follow = pg.PlotDataItem(pen=pg.mkPen('r', width=2, style=Qt.DashLine))
        lasso_path = pg.PlotDataItem(pen=pg.mkPen(color=(0, 255, 255), width=3, style=Qt.DashLine),
                                     symbolPen=(0, 255, 255), symbol='o', symbolSize=4)
        ruler_path = pg.PlotDataItem(pen=pg.mkPen(color='y', width=3, style=Qt.DashLine),
                                     symbolPen='y', symbolBrush='y', symbol='s', symbolSize=4)

        overlay_img = pg.ImageItem()
        overlay_img.setCompositionMode(QPainter.CompositionMode_Plus)
        overlay_contour = pg.ImageItem()
        overlay_contour.setLevels(levels=(0, 1))

        mask_img = pg.ImageItem()
        mask_img.setLevels(levels=(0, 1))

        tri_pnts = TriangulationPoints()
        tri_pnts.setVisible(False)
        self.tri_lines_list = []

        grid_lines = pg.GridItem(pen=(128, 128, 128))
        grid_lines.setVisible(False)

        probe_pnts = pg.ScatterPlotItem(pen=(0, 0, 255), brush=(0, 0, 255), symbol='s', size=5, hoverSize=8)
        cell_pnts = pg.ScatterPlotItem(pen=(0, 255, 0), brush=(0, 255, 0), size=5, hoverSize=8)
        blob_pnts = pg.ScatterPlotItem(pen=(55, 55, 55), brush=(55, 55, 55), symbol='s', size=5)
        drawing_pnts = pg.PlotDataItem(pen=pg.mkPen(color=(255, 102, 0), width=2), brush=None)
        contour_pnts = pg.PlotDataItem(pen=pg.mkPen(color=(255, 0, 255), width=3), brush=None)

        virus_img = pg.ImageItem()
        virus_img.setLevels(levels=(0, 1))

        self.image_list = [img1, img2, img3, img4]
        self.image_dict = {'img-overlay': overlay_img,
                           'overlay_contour': overlay_contour,
                           'img-virus': virus_img,
                           'img-contour': contour_pnts,
                           'img-mask': mask_img,
                           'grid_lines': grid_lines,
                           'tri_pnts': tri_pnts,
                           'circle_follow': circle_follow,
                           'lasso_path': lasso_path,
                           'ruler_path': ruler_path,
                           'img-probe': probe_pnts,
                           'img-cells': cell_pnts,
                           'img-blob': blob_pnts,
                           'img-drawing': drawing_pnts}
        self.image_dict_keys = list(self.image_dict.keys())

        self.vb.addItem(self.base_layer)
        for i in range(4):
            self.vb.addItem(self.image_list[i])
            self.image_list[i].setVisible(False)

        for i in range(len(self.image_dict)):
            self.vb.addItem(self.image_dict[self.image_dict_keys[i]])

    def set_data(self, data, scale=None):
        self.data = data
        if scale is not None:
            self.resetTransform()
            self.scale(*scale)
        base_img = np.zeros(data.shape[:2], 'uint8')
        self.base_layer.setImage(base_img)
        for i in range(self.data.shape[2]):
            self.image_list[i].setImage(self.data[:, :, i], autoLevels=False)
            self.image_list[i].setVisible(True)

    def set_lut(self, lut_list, bit_level):
        for i in range(self.data.shape[2]):
            self.image_list[i].setLevels(levels=(0, bit_level))
            self.image_list[i].setLookupTable(lut_list[i])

    def set_opacity(self, val):
        for i in range(len(self.data)):
            self.image_list[i].setOpts(opacity=val)

    def boundingRect(self):
        return self.base_layer.boundingRect()

    def mouse_hovered(self, event):
        if event.isExit():
            return
        try:
            pos = (event.pos())
            id = 1  # self.label_data[int(event.pos().x()), int(event.pos().y())]
        except (IndexError, AttributeError):
            return
        self.sig_mouse_hovered.emit(event)

    def mouse_clicked(self, pos):
        self.sig_mouse_clicked.emit(pos)

    def change_background_color(self, color: tuple):
        self.setBackground(color)

    def keyPressEvent(self, event):
        if event.key() == QtCore.Qt.Key_Backspace:
            print("Killing")
            self.sig_key_pressed.emit('delete')
        # elif event.key() == Qt.Key_Enter or event.key() == Qt.Key_Return:
        #     print('enter')
        #     self.sig_key_pressed.emit('enter')







import cv2
import numpy as np
import random


# Check if a point is inside a rectangle
def rect_contains(rect, point):
    if point[0] < rect[0]:
        return False
    elif point[1] < rect[1]:
        return False
    elif point[0] > rect[2]:
        return False
    elif point[1] > rect[3]:
        return False
    return True


# Draw a point
def draw_point(img, p, color):
    cv2.circle(img, p, 2, color, cv2.FILLED, cv2.LINE_AA, 0)


# Draw delaunay triangles
def draw_delaunay(img, subdiv, delaunay_color):
    triangleList = subdiv.getTriangleList()
    size = img.shape
    r = (0, 0, size[1], size[0])
    for t in triangleList:
        pt1 = (int(t[0]), int(t[1]))
        pt2 = (int(t[2]), int(t[3]))
        pt3 = (int(t[4]), int(t[5]))
        if rect_contains(r, pt1) and rect_contains(r, pt2) and rect_contains(r, pt3):
            cv2.line(img, pt1, pt2, delaunay_color, 1, cv2.LINE_AA, 0)
            cv2.line(img, pt2, pt3, delaunay_color, 1, cv2.LINE_AA, 0)
            cv2.line(img, pt3, pt1, delaunay_color, 1, cv2.LINE_AA, 0)


# Draw voronoi diagram
def draw_voronoi(img, subdiv):
    (facets, centers) = subdiv.getVoronoiFacetList([])

    for i in range(0, len(facets)):
        ifacet_arr = []
        for f in facets[i]:
            ifacet_arr.append(f)
        ifacet = np.array(ifacet_arr, 'i')
        color = (random.randint(0, 255), random.randint(0, 255), random.randint(0, 255))
        cv2.fillConvexPoly(img, ifacet, color, cv2.LINE_AA, 0)
        ifacets = np.array([ifacet])
        cv2.polylines(img, ifacets, True, (0, 0, 0), 1, cv2.LINE_AA, 0)
        cv2.circle(img, (int(centers[i][0]), int(centers[i][1])), 3, (0, 0, 0), cv2.FILLED, cv2.LINE_AA, 0)


if __name__ == '__main__':
    # Define window names
    win_delaunay = "Delaunay Triangulation"
    win_voronoi = "Voronoi Diagram"
    # Turn on animation while drawing triangles
    animate = True
    # Define colors for drawing.
    delaunay_color = (255, 255, 255)
    points_color = (0, 0, 255)
    # Read in the image.
    img = cv2.imread("/Users/jingyig/Work/Kavli/PyCode/herrbs/obama.jpg")
    # Keep a copy around
    img_orig = img.copy()
    # Rectangle to be used with Subdiv2D
    size = img.shape
    rect = (0, 0, size[1], size[0])
    # Create an instance of Subdiv2D
    subdiv = cv2.Subdiv2D(rect)
    # Create an array of points.
    points = []
    # Read in the points from a text file
    with open("/Users/jingyig/Work/Kavli/PyCode/herrbs/points.txt") as file:
        for line in file:
            x, y = line.split()
            points.append((int(x), int(y)))

    # Insert points into subdiv
    for p in points:
        subdiv.insert(p)
        # Show animation
        if animate:
            img_copy = img_orig.copy()
            # Draw delaunay triangles
            draw_delaunay(img_copy, subdiv, (255, 255, 255))
            cv2.imshow(win_delaunay, img_copy)
            cv2.waitKey(100)

    # Draw delaunay triangles
    draw_delaunay(img, subdiv, (255, 255, 255))

    # Draw points

    for p in points:
        draw_point(img, p, (0, 0, 255))

    # Allocate space for Voronoi Diagram
    img_voronoi = np.zeros(img.shape, dtype=img.dtype)
    # Draw Voronoi diagram
    draw_voronoi(img_voronoi, subdiv)
    # Show results
    cv2.imshow(win_delaunay, img)
    cv2.imshow(win_voronoi, img_voronoi)
    cv2.waitKey(0)



class Styles(object):
    def __init__(self):

        self.text_combo_list_style = '''
            QListView {
                background: #656565;
                border: 1px solid gray;
                color: white;
                border-radius: 0px;
            }
        
            QListView::item {
                border: None;
                background: transparent;
                margin:3px;
                height: 24px;
            }                             
        
            QListView::item:selected { 
                border: None;
                margin:3px;
                color: white;
                background: #232323; 
                height: 24px;
            }
        
            QListView::item:selected:!active {
                background: #323232;
                border: None;
            }
        
            QListView::item:selected:active {
                background: #323232;
                border: None;
            }
        
            QListView::item:hover {
                background: #323232;
                border: None;
            }
        '''

        self.text_combo_style = '''
            /*---------------------- QComboBox -----------------------*/
            QComboBox {
                subcontrol-origin: padding;
                subcontrol-position: top right;
                selection-background-color: transparent;
                color: white;
                background-color: transparent;
                border: None;
                border-radius: 5px;
                padding: 0px 0px 0px 5px;
                margin: 0px;
            }
            
            QComboBox:item {
                background: #323232;
                color: white;
                min-height: 10px;
                margin: 0px;
            }
            
            QComboBox:item:selected
            {
                border: None;
                background: #232323;
                margin: 0px;
            }
            
            
            QComboBox:editable {
                background: transparent;
            }
            
            QComboBox:!editable, QComboBox::drop-down:editable {
                 background: #656565;
            }
            
            /* QComboBox gets the "on" state when the popup is open */
            QComboBox:!editable:on, QComboBox::drop-down:editable:on {
                background: transparent;
            }
            
            QComboBox:on { /* shift the text when the popup opens */
                padding: 3px;
                color: white;
                background-color: transparent;
                selection-background-color: transparent;
            }
            
            QComboBox::drop-down {
                subcontrol-origin: padding;
                subcontrol-position: top right;
                width: 20px;
                border-top: None;
                border-bottom: None;
                border-left-width: 1px;
                border-left-color: transparent;
                border-left-style: solid; /* just a single line */
                border-top-right-radius: 3px; /* same radius as the QComboBox */
                border-bottom-right-radius: 3px;
            }
            
            QComboBox::down-arrow {
                image: url(icons/tdown.svg);
                width: 13px;
                height: 14px;
                padding-right: 3px;
            }
            
            QComboBox::down-arrow:on { /* shift the arrow when popup is open */
                top: 1px;
                left: 1px;
            }
            
            QComboBox QAbstractItemView {
                border: 2px solid darkgray;
            }
            
            QComboBox QListView {
                background: #656565;
                border: 1px solid gray;
                color: white;
                border-radius: 0px;
            }
        
            QComboBox QListView::item {
                border: None;
                background: transparent;
                margin:3px;
                height: 24px;
            }                             
        
            QComboBox QListView::item:selected { 
                border: None;
                margin:3px;
                color: white;
                background: #232323; 
                height: 24px;
            }
        
            QComboBox QListView::item:selected:!active {
                background: #323232;
                border: None;
            }
        
            QComboBox QListView::item:selected:active {
                background: #323232;
                border: None;
            }
        
            QComboBox QListView::item:hover {
                background: #323232;
                border: None;
            }

        '''

        self.tab_style = '''
            /*---------------------- QTabWidget -----------------------*/
            QTabWidget{
                background-color: rgb(50, 50, 50);
                margin-bottom: 0px;
    
            }
    
            QTabWidget::pane {
                background-color: transparent;
                margin-left: 0px;
                padding-left:0px;
                border-top: 1px solid #747a80;
                border-left: 1px solid #747a80;
                border-right: 1px solid #747a80;
                border-bottom: 1px solid #747a80;
                margin-bottom: 0px;
            }
    
            QTabWidget::tab-bar {
                bottom: 0;
                border: 1px solid gray;
            }
    
            QTabBar::tab {
                background-color: #3c3f41;
                height:30px;
                width: 30px;
                border-bottom: 3px solid #323232;
                margin: 0px;
                padding-top: 5px;
                padding-left: 6px;
            }
    
            QTabBar::tab:selected {
                background: #4e5254;
                border-bottom: 3px solid #747a80;
            }
    
            QTabBar::tab:hover{
                background: #27292a;
            }
    
            QGroupBox {
                background-color: transparent; 
                color: white;
                border: 1px solid; 
                border-radius: 5px; 
                padding-top: 3px; 
                padding-bottom: 3px; 
                margin-top: 0px
            }
    
            QLabel {
                color: white;
            }
    
            /* --------------------------------------- QScrollBar  -----------------------------------*/
            QScrollBar:horizontal
            {
                height: 15px;
                margin: 3px 15px 3px 15px;
                border: None;
                border-radius: 4px;
                background-color: transparent;
            }
    
            QScrollBar::handle:horizontal
            {
                background-color: #656565;
                min-width: 5px;
                border-radius: 4px;
            }
    
            QScrollBar::add-line:horizontal
            {
                margin: 0px 3px 0px 3px;
                width: 10px;
                height: 10px;
                subcontrol-position: right;
                subcontrol-origin: margin;
                border: None;
            }
    
            QScrollBar::sub-line:horizontal
            {
                margin: 0px 3px 0px 3px;
                height: 10px;
                width: 10px;
                subcontrol-position: left;
                subcontrol-origin: margin;
                border: None;
            }
    
            QScrollBar::add-line:horizontal:hover,QScrollBar::add-line:horizontal:on
            {
                height: 10px;
                width: 10px;
                subcontrol-position: right;
                subcontrol-origin: margin;
                border: None;
            }
    
    
            QScrollBar::sub-line:horizontal:hover, QScrollBar::sub-line:horizontal:on
            {
                height: 10px;
                width: 10px;
                subcontrol-position: left;
                subcontrol-origin: margin;
                border: None;
            }
    
            QScrollBar::up-arrow:horizontal, QScrollBar::down-arrow:horizontal
            {
                background: transparent;
            }
    
    
            QScrollBar::add-page:horizontal, QScrollBar::sub-page:horizontal
            {
                background: transparent;
            }
    
            QScrollBar:vertical
            {
                background-color: transparent;
                width: 15px;
                margin: 15px 3px 15px 3px;
                border: None;
                border-radius: 4px;
            }
    
            QScrollBar::handle:vertical
            {
                background-color: #656565;
                min-height: 5px;
                border-radius: 4px;
            }
    
            QScrollBar::sub-line:vertical
            {
                margin: 3px 0px 3px 0px;
                height: 10px;
                width: 10px;
                subcontrol-position: top;
                subcontrol-origin: margin;
                border: None;
            }
    
            QScrollBar::add-line:vertical
            {
                margin: 3px 0px 3px 0px;
                height: 10px;
                width: 10px;
                subcontrol-position: bottom;
                subcontrol-origin: margin;
                border: None;
            }
    
            QScrollBar::sub-line:vertical:hover,QScrollBar::sub-line:vertical:on
            {
                height: 10px;
                width: 10px;
                subcontrol-position: top;
                subcontrol-origin: margin;
                border: None;
            }
    
            QScrollBar::add-line:vertical:hover, QScrollBar::add-line:vertical:on
            {
                height: 10px;
                width: 10px;
                subcontrol-position: bottom;
                subcontrol-origin: margin;
                border: None;
            }
    
            QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical
            {
                background: transparent;
            }
    
            QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical
            {
                background: transparent;
            }
        '''

        self.sidebar_title_label_style = '''
            QLabel{
                color: white;
                background: #747a80;
                width: 300px;
                height: 20px;
                padding-bottom: 5px;
                padding-left: 5px;
                padding-top: 2px;
            }
        '''


import os
import sys
import numpy as np
import pyqtgraph as pg
from PyQt5.QtCore import *
from pyqtgraph.Qt import QtGui, QtCore


class MovablePoints(pg.GraphItem):
    mouseHovered = pyqtSignal(object)
    mouseDragged = pyqtSignal(object)
    mouseClicked = pyqtSignal(object)

    def __init__(self):
        self.dragPoint = None
        self.dragOffset = None
        pg.GraphItem.__init__(self)
        self.data = None
        self.y_range = None

    def setData(self, **kwds):
        self.data = kwds
        if 'pos' in self.data:
            if self.y_range is None:
                self.y_range = self.data['pos'][-1, 1]
            self.npts = self.data['pos'].shape[0]
            self.data['data'] = np.empty(self.npts, dtype=[('index', int)])
            self.data['data']['index'] = np.arange(self.npts)
        self.updateGraph()

    def updateGraph(self):
        pg.GraphItem.setData(self, **self.data)

    def mouseDragEvent(self, ev):
        if ev.button() != QtCore.Qt.LeftButton:
            ev.ignore()
            return

        if ev.isStart():
            pos = ev.buttonDownPos()
            pts = self.scatter.pointsAt(pos)
            if len(pts) == 0:
                # self.mouseClicked.emit(ev)
                ev.ignore()
                return
            self.dragPoint = pts[0]
            ind = pts[0].data()[0]
            if ind in [0, self.npts-1]:
                return
            self.dragOffset = self.data['pos'][ind] - pos
        elif ev.isFinish():
            self.dragPoint = None
            return
        else:
            if self.dragPoint is None:
                ev.ignore()
                return

        ind = self.dragPoint.data()[0]
        temp_pos = ev.pos()
        if temp_pos[0] <= self.data['pos'][ind - 1, 0]:
            temp_pos[0] = self.data['pos'][ind - 1, 0] + 1
        if temp_pos[0] >= self.data['pos'][ind + 1, 0]:
            temp_pos[0] = self.data['pos'][ind + 1, 0] - 1

        self.data['pos'][ind] = temp_pos + self.dragOffset
        self.data['pos'][self.data['pos'][:, 1] > self.y_range, 1] = self.y_range
        self.data['pos'][self.data['pos'][:, 1] < 0, 1] = 0
        self.updateGraph()
        self.mouseDragged.emit((ev, ind))
        ev.accept()

    def clear(self):
        self.scatter.clear()



class TriangulationPoints(pg.GraphItem):
    mouseHovered = pyqtSignal(object)
    mouseDragged = pyqtSignal(object)
    mouseClicked = pyqtSignal(object)

    def __init__(self):
        self.dragPoint = None
        self.dragOffset = None
        pg.GraphItem.__init__(self)
        self.scatter.sigClicked.connect(self.onclick)
        self.data = None
        self.y_range = None
        self.x_range = None

    def setData(self, **kwds):
        self.data = kwds
        if 'pos' in self.data:
            self.npts = self.data['pos'].shape[0]
            self.data['data'] = np.empty(self.npts, dtype=[('index', int)])
            self.data['data']['index'] = np.arange(self.npts)
        self.update_graph()

    def set_range(self, x_range, y_range):
        self.x_range = x_range
        self.y_range = y_range

    def mouseDragEvent(self, ev):
        if ev.button() != Qt.LeftButton:
            ev.ignore()
            return

        if ev.isStart():
            pos = ev.buttonDownPos()
            pts = self.scatter.pointsAt(pos)
            if len(pts) == 0:
                # self.mouseClicked.emit(ev)
                ev.ignore()
                return
            self.dragPoint = pts[0]
            ind = pts[0].data()[0]

            self.dragOffset = self.data['pos'][ind] - pos
        elif ev.isFinish():
            self.dragPoint = None
            return
        else:
            if self.dragPoint is None:
                ev.ignore()
                return

        ind = self.dragPoint.data()[0]
        temp_pos = ev.pos()
        if temp_pos[0] <= 0:
            temp_pos[0] = 0
        if temp_pos[0] >= self.x_range:
            temp_pos[0] = self.x_range
        if temp_pos[1] <= 0:
            temp_pos[1] = 0
        if temp_pos[1] >= self.y_range:
            temp_pos[1] = self.y_range

        self.data['pos'][ind] = temp_pos + self.dragOffset

        self.update_graph()
        self.mouseDragged.emit((ev, ind))
        ev.accept()

    def update_graph(self):
        pg.GraphItem.setData(self, **self.data)

    def onclick(self, ev, point):
        ind = point[0].data()[0]
        print(ind)
        self.mouseClicked.emit((ev, ind))





class TriangulationPointsTest(pg.ScatterPlotItem):
    mouseHovered = pyqtSignal(object)
    mouseDragged = pyqtSignal(object)
    mouseClicked = pyqtSignal(object)

    def __init__(self):

        pg.ScatterPlotItem.__init__(self)
        self.sigClicked.connect(self.onclick)

        self.data = None

        self.y_range = None
        self.x_range = None

        self.drag_point = None
        self.drag_offset = None

        dtype = [
            ('x', float),
            ('y', float),
            ('size', float),
            ('symbol', object),
            ('pen', object),
            ('brush', object),
            ('visible', bool),
            ('data', object),
            ('hovered', bool),
            ('item', object),
            ('index', int),
            ('sourceRect', [
                ('x', int),
                ('y', int),
                ('w', int),
                ('h', int)
            ])
        ]

        self.data = np.empty(0, dtype=dtype)

    def setData(self, **kwds):
        self.data = kwds
        if 'pos' in self.data:
            self.npts = self.data['pos'].shape[0]
            self.data['data'] = np.empty(self.npts, dtype=[('index', int)])
            self.data['data']['index'] = np.arange(self.npts)
        # self.update_graph()

    def set_range(self, x_range, y_range):
        self.x_range = x_range
        self.y_range = y_range

    def mouseDragEvent(self, ev):
        if ev.button() != Qt.LeftButton:
            ev.ignore()
            return

        if ev.isStart():
            pos = ev.buttonDownPos()
            pts = self.pointsAt(pos)
            if len(pts) == 0:
                # self.mouseClicked.emit(ev)
                ev.ignore()
                return
            self.dragPoint = pts[0]
            ind = pts[0].data()[0]

            self.drag_offset = self.data['pos'][ind] - pos
        elif ev.isFinish():
            self.drag_point = None
            return
        else:
            if self.drag_point is None:
                ev.ignore()
                return

        ind = self.dragPoint.data()[0]
        temp_pos = ev.pos()
        if temp_pos[0] <= 0:
            temp_pos[0] = 0
        if temp_pos[0] >= self.x_range:
            temp_pos[0] = self.x_range
        if temp_pos[1] <= 0:
            temp_pos[1] = 0
        if temp_pos[1] >= self.y_range:
            temp_pos[1] = self.y_range

        self.data['pos'][ind] = temp_pos + self.dragOffset

        self.setData(pos=self.data['pos'])
        self.mouseDragged.emit((ev, ind))
        ev.accept()

    # def update_graph(self):
    #     pg.ScatterPlotItem.setData(self, **self.data)

    def onclick(self, ev, point):
        ind = point[0].data()[0]
        print(ind)
        self.mouseClicked.emit((ev, ind))




import os
from os.path import dirname, realpath, join
import sys
from sys import argv, exit
from pathlib import Path

import pickle
import csv
import nibabel as nib
import numpy as np
import pandas as pd
import cv2
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

from .uuuuuu import make_contour_img, render_volume, render_small_volume
from .atlas_loader import process_atlas_raw_data, AtlasLoader


class AtlasProcessor(QDialog):
    def __init__(self):
        super().__init__()
        self.setStyleSheet('color: black;')
        self.setWindowTitle("Atlas Processor")

        self.folder_path = None
        self.data_local = None
        self.segmentation_local = None
        self.mask_local = None
        self.labinf_path = None
        self.bregma_coord = [0, 0, 0]
        self.lambda_coord = [0, 0, 0]
        self.voxel_size = 0
        self.factor_val = 2

        layout = QGridLayout(self)

        data_label = QLabel('Volume File:')
        self.data_btn = QPushButton('Select File')
        self.data_line = QLineEdit()

        seg_label = QLabel('Segmentation File: ')
        self.seg_btn = QPushButton('Select File')
        self.seg_line = QLineEdit()

        mask_label = QLabel('Mask File (optional): ')
        self.mask_btn = QPushButton('Select File')
        self.mask_line = QLineEdit()

        labinf_label = QLabel('Label Information File:')
        self.labinf_btn = QPushButton('Select File')
        self.labinf_line = QLineEdit()

        valid_input = QIntValidator(0, 99999)

        float_input = QRegExpValidator(QRegExp(r'[0-9].+'))
        # float_input = QDoubleValidator(0.0, 100.0, 6)

        bregma_label = QLabel('Bregma Coordinates (voxel): ')
        self.bregma_input1 = QLineEdit('0')
        self.bregma_input1.setValidator(valid_input)
        self.bregma_input2 = QLineEdit('0')
        self.bregma_input2.setValidator(valid_input)
        self.bregma_input3 = QLineEdit('0')
        self.bregma_input3.setValidator(valid_input)

        lambda_label = QLabel('Lambda Coordinates (voxel): ')
        self.lambda_input1 = QLineEdit('0')
        self.lambda_input1.setValidator(valid_input)
        self.lambda_input2 = QLineEdit('0')
        self.lambda_input2.setValidator(valid_input)
        self.lambda_input3 = QLineEdit('0')
        self.lambda_input3.setValidator(valid_input)

        vox_size_label = QLabel('Voxel Size (um): ')
        self.vox_size_input1 = QLineEdit('0')
        self.vox_size_input1.setValidator(float_input)

        factor_label = QLabel('Factor (voxel): ')
        self.factor_input1 = QLineEdit('2')
        self.factor_input1.setValidator(valid_input)

        selector_vals = ['Left --> Right', 'Right --> Left',
                         'Superior --> Inferior', 'Inferior --> Superior',
                         'Posterior --> Anterior', 'Anterior --> Posterior']

        # x_axis_label = QLabel('x-axis: ')
        # self.x_axis_combo = QComboBox()
        # self.x_axis_combo.addItems(selector_vals)
        #
        # y_axis_label = QLabel('y-axis: ')
        # self.y_axis_combo = QComboBox()
        # self.y_axis_combo.addItems(selector_vals)
        #
        # z_axis_label = QLabel('z-axis: ')
        # self.z_axis_combo = QComboBox()
        # self.z_axis_combo.addItems(selector_vals)

        self.process_btn = QPushButton('Start Process')

        layout.addWidget(data_label, 0, 0, 1, 1)
        layout.addWidget(self.data_btn, 0, 1, 1, 1)
        layout.addWidget(self.data_line, 0, 2, 1, 2)
        layout.addWidget(seg_label, 1, 0, 1, 1)
        layout.addWidget(self.seg_btn, 1, 1, 1, 1)
        layout.addWidget(self.seg_line, 1, 2, 1, 2)
        layout.addWidget(mask_label, 2, 0, 1, 1)
        layout.addWidget(self.mask_btn, 2, 1, 1, 1)
        layout.addWidget(self.mask_line, 2, 2, 1, 2)
        layout.addWidget(labinf_label, 3, 0, 1, 1)
        layout.addWidget(self.labinf_btn, 3, 1, 1, 1)
        layout.addWidget(self.labinf_line, 3, 2, 1, 2)
        layout.addWidget(bregma_label, 4, 0, 1, 1)
        layout.addWidget(self.bregma_input1, 4, 1, 1, 1)
        layout.addWidget(self.bregma_input2, 4, 2, 1, 1)
        layout.addWidget(self.bregma_input3, 4, 3, 1, 1)
        layout.addWidget(lambda_label, 5, 0, 1, 1)
        layout.addWidget(self.lambda_input1, 5, 1, 1, 1)
        layout.addWidget(self.lambda_input2, 5, 2, 1, 1)
        layout.addWidget(self.lambda_input3, 5, 3, 1, 1)
        layout.addWidget(vox_size_label, 6, 0, 1, 1)
        layout.addWidget(self.vox_size_input1, 6, 1, 1, 1)
        layout.addWidget(factor_label, 6, 2, 1, 1)
        layout.addWidget(self.factor_input1, 6, 3, 1, 1)

        # layout.addWidget(x_axis_label, 6, 0, 1, 1)
        # layout.addWidget(self.x_axis_combo, 6, 1, 1, 3)
        # layout.addWidget(y_axis_label, 7, 0, 1, 1)
        # layout.addWidget(self.y_axis_combo, 7, 1, 1, 3)
        # layout.addWidget(z_axis_label, 8, 0, 1, 1)
        # layout.addWidget(self.z_axis_combo, 8, 1, 1, 3)

        layout.addWidget(self.process_btn, 7, 0, 1, 4)

        # connect all buttons
        self.data_btn.clicked.connect(self.get_data_file)
        self.seg_btn.clicked.connect(self.get_seg_file)
        self.mask_btn.clicked.connect(self.get_mask_file)
        self.labinf_btn.clicked.connect(self.get_info_file)
        self.process_btn.clicked.connect(self.process_data)
        self.bregma_input1.textChanged.connect(self.bregma_input1_changed)
        self.bregma_input2.textChanged.connect(self.bregma_input2_changed)
        self.bregma_input3.textChanged.connect(self.bregma_input3_changed)
        self.lambda_input1.textChanged.connect(self.lambda_input1_changed)
        self.lambda_input2.textChanged.connect(self.lambda_input2_changed)
        self.lambda_input3.textChanged.connect(self.lambda_input3_changed)
        self.vox_size_input1.textChanged.connect(self.vox_size_input_changed)
        self.factor_input1.textChanged.connect(self.factor_input_changed)

    def get_folder_path(self, file_path):
        self.folder_path = os.path.dirname(file_path)

    def get_data_file(self):
        if self.folder_path is not None:
            data_path = QFileDialog.getOpenFileName(self, 'Select Atlas Volume File', self.folder_path)
        else:
            data_path = QFileDialog.getOpenFileName(self, 'Select Atlas Volume File')
            self.get_folder_path(data_path[0])
        self.data_local = os.path.basename(data_path[0])
        self.data_line.setText(self.data_local)

    def get_seg_file(self):
        if self.folder_path is not None:
            seg_path = QFileDialog.getOpenFileName(self, 'Select Segmentation File', self.folder_path)
        else:
            seg_path = QFileDialog.getOpenFileName(self, 'Select Segmentation File')
            self.get_folder_path(seg_path[0])
        self.segmentation_local = os.path.basename(seg_path[0])
        self.seg_line.setText(self.segmentation_local)

    def get_mask_file(self):
        if self.folder_path is not None:
            mask_path = QFileDialog.getOpenFileName(self, 'Select Mask File', self.folder_path)
        else:
            mask_path = QFileDialog.getOpenFileName(self, 'Select Mask File')
            self.get_folder_path(mask_path[0])
        self.mask_local = os.path.basename(mask_path[0])
        self.mask_line.setText(self.mask_local)

    def get_info_file(self):
        if self.folder_path is not None:
            self.info_path = QFileDialog.getOpenFileName(self, 'Select Label Information File', self.folder_path)
        else:
            self.info_path = QFileDialog.getOpenFileName(self, 'Select Label Information File')
            self.get_folder_path(self.info_path[0])
        self.labinf_line.setText(os.path.basename(self.info_path[0]))

    def bregma_input1_changed(self, text):
        if text == '':
            return
        self.bregma_coord[0] = int(text)

    def bregma_input2_changed(self, text):
        if text == '':
            return
        self.bregma_coord[1] = int(text)

    def bregma_input3_changed(self, text):
        if text == '':
            return
        self.bregma_coord[2] = int(text)

    def lambda_input1_changed(self, text):
        if text == '':
            return
        self.lambda_coord[0] = int(text)

    def lambda_input2_changed(self, text):
        if text == '':
            return
        self.lambda_coord[1] = int(text)

    def lambda_input3_changed(self, text):
        if text == '':
            return
        self.lambda_coord[2] = int(text)

    def vox_size_input_changed(self, text):
        if text == '':
            return
        self.voxel_size = float(text)

    def factor_input_changed(self, text):
        if text == '':
            return
        self.factor_val = int(text)

    def process_data(self):
        if self.data_local is None or self.segmentation_local is None:
            msg = 'Please give the atlas volume file and label segmentation file.'
            return
        if self.voxel_size == 0 or self.factor_val == 0:
            msg = 'Please give the correct voxel size. Voxel size can not be 0.'
            return

        da_atlas = AtlasLoader(self.folder_path)
        if da_atlas.success:
            atlas_data = da_atlas.atlas_data
            segmentation_data = da_atlas.segmentation_data
            msg = 'Atlas loaded successfully.'
        else:
            atlas_data, atlas_info, segmentation_data, boundary, msg = \
                process_atlas_raw_data(self.folder_path, data_file=self.data_local,
                                       segmentation_file=self.segmentation_local, mask_file=self.mask_local,
                                       bregma_coordinates=self.bregma_coord, lambda_coordinates=self.lambda_coord,
                                       voxel_size=self.voxel_size)

        if msg == 'Atlas loaded successfully.':
            #render_volume(atlas_data, self.folder_path, factor=self.factor_val, level=0.1)
            # render_small_volume(atlas_data, segmentation_data, self.folder_path, factor=self.factor_val, level=0.1)
            self.process_btn.setVisible(False)

            self.close()
        else:
            return msg






import time
import os
import sys
from os.path import dirname, realpath, join
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *
import pyqtgraph.opengl as gl

import nrrd
import pickle
import shutil
import requests
import numpy as np
import pandas as pd

from .atlas_loader import process_atlas_raw_data
from .uuuuuu import render_volume, render_small_volume, hex2rgb, obj_data_to_mesh3d, make_atlas_label_contour
from .atlas_downloader import DownloadThread


class WorkerProcessAllen(QObject):
    finished = pyqtSignal()
    progress = pyqtSignal(int)

    def __init__(self):
        super(WorkerProcessAllen, self).__init__()
        self.saving_folder = None
        self.data_local = None
        self.segmentation_local = None
        self.label_local = None
        self.b_val = None
        self.l_val = None
        self.vox_size = None

        self.atlas_data = None
        self.atlas_info = None
        self.segmentation_data = None
        self.unique_label = None
        self.label_info = None
        self.boundary = None
        self.small_mesh_list = {}
        self.mesh_data = None

    def set_data(self, saving_folder, data_local, segmentation_local, label_local, b_val, vox_size):
        self.saving_folder = saving_folder
        self.data_local = data_local
        self.segmentation_local = segmentation_local
        self.label_local = label_local
        self.b_val = b_val
        self.vox_size = vox_size

    def run(self):
        label_data, header = nrrd.read(os.path.join(self.saving_folder, self.segmentation_local))
        self.progress.emit(5)
        self.unique_label = np.unique(label_data)
        self.progress.emit(9)

        n_unique_labels = len(self.unique_label)
        mesh_path = os.path.join(self.saving_folder, 'meshes')
        if not os.path.exists(mesh_path):
            os.mkdir(mesh_path)

        self.progress.emit(10)

        atlas_size = label_data.shape

        downloaded_mesh_path = os.path.join(self.saving_folder, 'downloaded_meshes')

        progress_step = np.linspace(10, 40, n_unique_labels)
        for i in range(n_unique_labels):
            ind = self.unique_label[i]
            self.progress.emit(progress_step[i])
            if ind in [545, 0]:
                continue

            filename = os.path.join(downloaded_mesh_path, '{}.obj'.format(ind))

            vertices, faces = obj_data_to_mesh3d(filename)
            vertices = vertices / self.vox_size

            vertices[:, 0] = atlas_size[0] - vertices[:, 0]
            vertices[:, 1] = atlas_size[1] - vertices[:, 1]

            verts = vertices.copy()
            verts[:, 0] = vertices[:, 2].copy()
            verts[:, 1] = vertices[:, 0].copy()
            verts[:, 2] = vertices[:, 1].copy()

            md = gl.MeshData(vertexes=verts, faces=faces)

            outfile = open(os.path.join(mesh_path, '{}.pkl'.format(ind)), 'wb')
            pickle.dump(md, outfile)
            outfile.close()

        file_list = os.listdir(mesh_path)
        for da_file in file_list:
            file_name = os.path.basename(da_file)
            da_name, file_extension = os.path.splitext(file_name)
            if file_extension == '.pkl':
                infile = open(os.path.join(mesh_path, da_file), 'rb')
                md = pickle.load(infile)
                infile.close()

                self.small_mesh_list[str(da_name)] = md

        outfile = open(os.path.join(self.saving_folder, 'atlas_small_meshdata.pkl'), 'wb')
        pickle.dump(self.small_mesh_list, outfile)
        outfile.close()

        target = os.path.join(self.saving_folder, "atlas_meshdata.pkl")
        shutil.copyfile(join(mesh_path, '997.pkl'), target)

        infile = open(os.path.join(self.saving_folder, 'atlas_meshdata.pkl'), 'rb')
        self.mesh_data = pickle.load(infile)
        infile.close()

        self.progress.emit(50)

        df = pd.read_csv(os.path.join(self.saving_folder, self.label_local))

        da_labels = df['safe_name'].values
        for i in range(len(da_labels)):
            if da_labels[i] == 'root':
                da_labels[i] = 'Brain'

        da_short_label = df['acronym'].values
        for i in range(len(da_short_label)):
            if da_short_label[i] == 'root':
                da_short_label[i] = 'Brain'

        hex_colors = df['color_hex_triplet'].values
        rgb_colors = []
        for i in range(len(hex_colors)):
            r, g, b = hex2rgb(hex_colors[i])
            rgb_colors.append([r, g, b])
        rgb_colors = np.asarray(rgb_colors)

        levels = []
        structure_id_path = df['structure_id_path'].values
        for i in range(len(structure_id_path)):
            da_path = structure_id_path[i]
            da_path_split = da_path.split('/')
            for j in np.arange(len(da_path_split))[::-1]:
                if da_path_split[j] == '':
                    da_path_split.pop(j)
            levels.append(len(da_path_split))

        self.label_info = {'index': df['id'].values.astype(int),
                           'label': da_labels,
                           'parent': df['parent_structure_id'].values.astype(int),
                           'abbrev': da_short_label,
                           'color': rgb_colors,
                           'level_indicator': levels}

        with open(os.path.join(self.saving_folder, 'atlas_labels.pkl'), 'wb') as handle:
            pickle.dump(self.label_info, handle, protocol=pickle.HIGHEST_PROTOCOL)

        self.progress.emit(55)

        volume_data, header = nrrd.read(os.path.join(self.saving_folder, self.data_local))

        print(volume_data.shape)
        volume_data = np.transpose(volume_data[::-1, ::-1, :], (2, 0, 1))

        self.atlas_data = volume_data.copy()
        self.atlas_data = self.atlas_data - np.min(self.atlas_data)
        self.atlas_data = self.atlas_data / np.max(self.atlas_data)

        self.progress.emit(60)

        b_val = self.b_val.copy()
        print(b_val)
        if self.b_val[0] == 0:
            b_val[0] = int(atlas_size[0] / 2)
        if self.b_val[2] == 0:
            b_val[2] = int(atlas_size[2] / 2)

        self.progress.emit(61)

        self.atlas_info = [
            {'name': 'anterior', 'values': np.arange(self.atlas_data.shape[0]) * self.vox_size, 'units': 'um'},
            {'name': 'dorsal', 'values': np.arange(self.atlas_data.shape[1]) * self.vox_size, 'units': 'um'},
            {'name': 'right', 'values': np.arange(self.atlas_data.shape[2]) * self.vox_size, 'units': 'um'},
            {'vxsize': self.vox_size,
             'Bregma': [b_val[2], atlas_size[0] - 1 - b_val[0], atlas_size[1] - 1 - b_val[1]]}
        ]

        self.progress.emit(62)

        atlas = {'data': self.atlas_data, 'info': self.atlas_info}

        outfile = open(os.path.join(self.saving_folder, 'atlas_pre_made.pkl'), 'wb')
        pickle.dump(atlas, outfile)
        outfile.close()

        self.progress.emit(65)

        self.segmentation_data = np.transpose(label_data[::-1, ::-1, :], (2, 0, 1))
        self.segmentation_data = self.segmentation_data.astype(int)
        print(self.segmentation_data.shape)

        self.progress.emit(67)

        segment = {'data': self.segmentation_data, 'unique_label': self.unique_label}

        outfile = open(os.path.join(self.saving_folder, 'segment_pre_made.pkl'), 'wb')
        pickle.dump(segment, outfile)
        outfile.close()

        self.progress.emit(70)

        self.boundary = make_atlas_label_contour(self.saving_folder, self.segmentation_data)

        # target = os.path.join(self.saving_folder, 'atlas_labels.pkl')
        # if not os.path.exists(target):
        #     shutil.copyfile(join(dirname(__file__), "data/atlas_labels.pkl"), target)

        self.progress.emit(100)

        self.finished.emit()


class MeshDownloader(QObject):
    finished = pyqtSignal()
    progress = pyqtSignal(int)

    def __init__(self):
        super(MeshDownloader, self).__init__()

        self.save_folder = None
        self.segmentation_local = None

    def set_data(self, save_folder, segmentation_local):
        self.save_folder = save_folder
        self.segmentation_local = os.path.join(save_folder, segmentation_local)

    def run(self):
        label_data, header = nrrd.read(self.segmentation_local)
        unique_label = np.unique(label_data)

        n_unique_labels = len(unique_label)
        progress_step = np.linspace(0, 100, n_unique_labels)

        downloaded_mesh_path = os.path.join(self.save_folder, 'downloaded_meshes')
        if not os.path.exists(downloaded_mesh_path):
            os.mkdir(downloaded_mesh_path)

        for i in range(n_unique_labels):
            ind = unique_label[i]
            self.progress.emit(progress_step[i])
            if ind == 0:
                continue
            url = 'http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/annotation/ccf_2017/structure_meshes/{}.obj'.format(
                ind)
            r = requests.get(url, allow_redirects=True)
            da_file = open(os.path.join(downloaded_mesh_path, '{}.obj'.format(ind)), 'wb')
            da_file.write(r.content)
            da_file.close()
        self.finished.emit()


class AllenDownloader(QDialog):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        layout = QVBoxLayout(self)

        self.setWindowTitle("Allen Mice Atlas Downloader")

        self.thread = QThread()
        self.worker = WorkerProcessAllen()
        self.mesh_thread = QThread()
        self.mesh_worker = MeshDownloader()

        self.continue_process = True

        self.voxel_size = 10
        self.bregma_coord = [0, 0, 0]
        radio_group = QFrame()
        radio_group.setStyleSheet('QFrame{border: 1px solid gray; border-radius: 3px}')
        radio_group_layout = QHBoxLayout(radio_group)
        radio_group_layout.setContentsMargins(5, 0, 5, 0)
        radio_group_layout.setAlignment(Qt.AlignCenter)
        self.vs_rabnt1 = QRadioButton('10 um')
        self.vs_rabnt1.setChecked(True)
        self.vs_rabnt2 = QRadioButton('25 um')
        self.vs_rabnt3 = QRadioButton('50 um')
        radio_group_layout.addWidget(self.vs_rabnt1)
        radio_group_layout.addWidget(self.vs_rabnt2)
        radio_group_layout.addWidget(self.vs_rabnt3)

        self.vs_rabnt1.toggled.connect(self.voxel_size_radio_clicked)
        self.vs_rabnt2.toggled.connect(self.voxel_size_radio_clicked)
        self.vs_rabnt3.toggled.connect(self.voxel_size_radio_clicked)

        self.data_url = "http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/average_template/average_template_10.nrrd"
        self.segmentation_url = "http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/annotation/ccf_2017/annotation_10.nrrd"

        self.label_local = "query.csv"
        # self.data_local = "average_template_10.nrrd"
        # self.segmentation_local = "annotation_10.nrrd"
        self.data_local = "average_template_25.nrrd"
        self.segmentation_local = "annotation_25.nrrd"

        self.saving_folder = None

        self.finish = [False, False, False]
        self.process_finished = False

        self.label_bar = QProgressBar()
        self.label_bar.setMinimumWidth(400)
        self.label_bar.setValue(0)

        self.data_bar = QProgressBar()
        self.data_bar.setMinimumWidth(400)
        self.data_bar.setValue(0)

        self.segmentation_bar = QProgressBar()
        self.segmentation_bar.setMinimumWidth(400)
        self.segmentation_bar.setValue(0)

        self.mesh_bar = QProgressBar()
        self.mesh_bar.setMinimumWidth(400)
        self.mesh_bar.setValue(0)

        self.download_btn = QPushButton()
        self.download_btn. setMinimumWidth(100)
        self.download_btn.setText("Download")

        self.download_mesh_btn = QPushButton()
        self.download_mesh_btn.setMinimumWidth(100)
        self.download_mesh_btn.setText("Download Meshes")

        valid_input = QIntValidator(0, 99999)

        b_wrap = QFrame()
        b_layout = QHBoxLayout(b_wrap)

        b_label = QLabel('Bregma Coordinates (voxel): ')
        self.b_input1 = QLineEdit('0')
        self.b_input1.setStyleSheet('color: black')
        self.b_input1.setValidator(valid_input)
        self.b_input2 = QLineEdit('0')
        self.b_input2.setStyleSheet('color: black')
        self.b_input2.setValidator(valid_input)
        self.b_input3 = QLineEdit('0')
        self.b_input3.setStyleSheet('color: black')
        self.b_input3.setValidator(valid_input)

        b_layout.addWidget(b_label)
        b_layout.addWidget(self.b_input1)
        b_layout.addWidget(self.b_input2)
        b_layout.addWidget(self.b_input3)

        self.process_btn = QPushButton()
        self.process_btn.setMinimumWidth(100)
        self.process_btn.setText("Process")

        self.process_info = QLabel('The whole process takes some time. \n'
                                   'This window will be closed automatically when processing finished.')

        self.progress = QProgressBar(self)
        self.progress.setMinimumWidth(100)

        # ok button, used to close window
        ok_btn = QDialogButtonBox(QDialogButtonBox.Ok)
        ok_btn.accepted.connect(self.accept)

        layout.addWidget(radio_group)
        # layout.addWidget(self.label_bar)
        layout.addWidget(self.data_bar)
        layout.addWidget(self.segmentation_bar)
        layout.addWidget(self.download_btn)
        layout.addWidget(self.mesh_bar)
        layout.addWidget(self.download_mesh_btn)
        layout.addWidget(b_wrap)
        layout.addWidget(self.process_info)
        layout.addWidget(self.process_btn)
        layout.addWidget(self.progress)
        # layout.addWidget(ok_btn)

        # Binding Button Event
        self.download_btn.clicked.connect(self.download_start)
        self.download_mesh_btn.clicked.connect(self.download_mesh_start)
        self.process_btn.clicked.connect(self.process_start)

        self.b_input1.textChanged.connect(self.bregma_input1_changed)
        self.b_input2.textChanged.connect(self.bregma_input2_changed)
        self.b_input3.textChanged.connect(self.bregma_input3_changed)

    def bregma_input1_changed(self, text):
        if text == '':
            return
        self.bregma_coord[0] = int(text)

    def bregma_input2_changed(self, text):
        if text == '':
            return
        self.bregma_coord[1] = int(text)

    def bregma_input3_changed(self, text):
        if text == '':
            return
        self.bregma_coord[2] = int(text)

    def voxel_size_radio_clicked(self):
        if self.vs_rabnt1.isChecked():
            self.voxel_size = 10
            self.data_url = "http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/average_template/average_template_10.nrrd"
            self.segmentation_url = "http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/annotation/ccf_2017/annotation_10.nrrd"
            self.data_local = "average_template_10.nrrd"
            self.segmentation_local = "annotation_10.nrrd"
        elif self.vs_rabnt2.isChecked():
            self.voxel_size = 25
            self.data_url = 'http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/average_template/average_template_25.nrrd'
            self.segmentation_url = 'http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/annotation/ccf_2017/annotation_25.nrrd'
            self.data_local = "average_template_25.nrrd"
            self.segmentation_local = "annotation_25.nrrd"
        else:
            self.voxel_size = 50
            self.data_url = 'http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/average_template/average_template_50.nrrd'
            self.segmentation_url = 'http://download.alleninstitute.org/informatics-archive/current-release/mouse_ccf/annotation/ccf_2017/annotation_50.nrrd'
            self.data_local = "average_template_50.nrrd"
            self.segmentation_local = "annotation_50.nrrd"

    # Download button event
    def download_start(self):
        self.saving_folder = str(QFileDialog.getExistingDirectory(self, "Select Folder to Save Atlas"))

        if self.saving_folder != '':
            self.download_btn.setVisible(False)
            # target = os.path.join(self.saving_folder, 'atlas_labels.pkl')
            # if not os.path.exists(target):
            #     shutil.copyfile(join(dirname(__file__), "data/allen_mice_atlas_labels.pkl"), target)

            target = os.path.join(self.saving_folder, self.label_local)
            if not os.path.exists(target):
                shutil.copyfile(join(dirname(__file__), "data/query.csv"), target)

            self.start_thread(self.segmentation_url, self.segmentation_local, self.set_segmentation_bar_value)
            time.sleep(0.1)
            self.start_thread(self.data_url, self.data_local, self.set_data_bar_value)
            time.sleep(0.1)

    def download_mesh_start(self):
        if not self.finish[0] or not self.finish[1]:
            return
        self.download_mesh_btn.setVisible(False)
        if self.saving_folder is not None:
            saving_folder = self.saving_folder
        else:
            saving_folder = str(QFileDialog.getExistingDirectory(self, "Select Atlas Folder"))

        if saving_folder != '':
            self.mesh_worker.set_data(self.saving_folder, self.segmentation_local)
            self.mesh_worker.moveToThread(self.mesh_thread)
            self.mesh_thread.started.connect(self.mesh_worker.run)
            self.mesh_worker.finished.connect(self.mesh_thread.quit)
            self.mesh_worker.finished.connect(self.mesh_worker.deleteLater)
            self.mesh_thread.finished.connect(self.mesh_thread.deleteLater)
            self.mesh_worker.progress.connect(self.mesh_report_progress)
            self.mesh_thread.start()

    #
    def start_thread(self, url, local, func):
        file_size = requests.get(url, stream=True).headers['Content-Length']
        file_obj = open(os.path.join(self.saving_folder, local), 'wb')

        self.da_thread = DownloadThread(url, file_size, file_obj, buffer=1024)
        self.da_thread.download_process_signal.connect(func)
        self.da_thread.start()

    # Setting progress bar
    def set_data_bar_value(self, value):
        self.data_bar.setValue(value)
        if value == 100:
            self.finish[0] = True
            return

    def set_segmentation_bar_value(self, value):
        self.segmentation_bar.setValue(value)
        if value == 100:
            self.finish[1] = True
            return

    def report_progress(self, i):
        self.progress.setValue(i)

    def mesh_report_progress(self, i):
        self.mesh_bar.setValue(i)
        if i == 100:
            self.finish[2] = True
            return

    def process_start(self):
        if self.saving_folder is not None:
            saving_folder = self.saving_folder
        else:
            saving_folder = str(QFileDialog.getExistingDirectory(self, "Select Atlas Folder"))

        if saving_folder != '':
            self.process_btn.setVisible(False)
            self.worker.set_data(saving_folder, self.data_local, self.segmentation_local, self.label_local,
                                 self.bregma_coord, self.voxel_size)
            self.worker.moveToThread(self.thread)
            self.thread.started.connect(self.worker.run)
            self.worker.finished.connect(self.on_finish)
            # self.worker.finished.connect(self.worker.deleteLater)
            self.thread.finished.connect(self.thread.deleteLater)
            self.worker.progress.connect(self.report_progress)
            self.thread.start()

    def on_finish(self):
        self.thread.quit()
        self.close()

    def closeEvent(self, event):
        reply = QMessageBox.question(self, 'Message',
                                     "Do you want to leave?", QMessageBox.Yes, QMessageBox.No)

        if reply == QMessageBox.Yes:
            self.continue_process = False
            event.accept()
        else:
            event.ignore()






import os
import sys
import numpy as np
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *
import pyqtgraph as pg
from pyqtgraph.Qt import QtGui, QtCore

label_tree_style = '''
QWidget{
    background-color: transparent;
    color: white;
}

QColorDialog{
    background-color: rgb(50, 50, 50);
}

QPushButton{
    background: transparent;
    border-radius: 3px;
    color: white;
    border: None;
    height: 24px;
    margin: 5px;
    padding: 0px;
}

QTreeWidget{
    background-color: transparent;
    border: 1px solid #636363;
}

QTreeWidget::item{
    background-color: transparent;
    color: rgb(255, 255, 255);
    min-height: 20px;
}

QTreeWidget::item:selected{
    background-color: transparent;
    color: white;
}

QTreeView {
    border: 1px solid rgb(128, 128, 128);
    background-color: transparent;
    selection-background-color: transparent; /* Used on Mac */
    selection-color: white; /* Used on Mac */
    show-decoration-selected: 1;
}

QTreeView::item {
    border: None;
    background-color: transparent;
}

QTreeView::item:hover {
    background: transparent;
    border: None;
}

QTreeView::item:selected {
    border: None;
    background: transparent;
    color: white;
}

QTreeView::indicator:unchecked {
    border: 1px solid #656565;
    border-radius: 5px;
}

QTreeView::indicator:checked {
    border: 1px solid #656565;
    border-radius: 5px;
    image: url(icons/sidebar/tree_checked.svg);
}

QTreeView::branch {
        background: transparent;
}

QTreeView::branch:selected {
        background: transparent;
}

QTreeView::branch:selected:active {
    background: transparent;
}

QTreeView::branch:selected:!active {
    background: transparent;
}
QTreeView::branch:has-siblings:!adjoins-item {
    border-image: None;
    background: transparent;
}

QTreeView::branch:has-siblings:adjoins-item {
    border-image: None;
    background: transparent;
}

QTreeView::branch:!has-siblings:!adjoins-item {
    border-image: None;
    background: transparent;
}

QTreeView::branch:!has-siblings:adjoins-item {
    border-image: None;
    background: transparent;
}

QTreeView::branch:!has-children:!has-siblings:adjoins-item {
    border-image: None;
    background: transparent;
}

QTreeView::branch:has-children:!has-siblings:closed,
QTreeView::branch:closed:has-children:has-siblings {
        background: transparent;
        border-image: none;
        image: url(icons/tree_close.svg);
}

QTreeView::branch:open:has-children:!has-siblings,
QTreeView::branch:open:has-children:has-siblings  {
        border-image: none;
        background: transparent;
        image: url(icons/tree_open.svg);
}


'''

reset_button_style = '''
QPushButton{
    background: #656565;
    border-radius: 5px;
    color: white;
    border-style: outset;
    border-bottom: 1px solid rgb(30, 30, 30);
    min-height: 16px;
    margin: 0px;
}

QPushButton:hover{
    background-color: #323232;
    border: 1px solid #656565;
}

'''


class SignalBlock(object):
    """Class used to temporarily block a Qt signal connection::

        with SignalBlock(signal, slot):
            # do something that emits a signal; it will
            # not be delivered to slot
    """
    def __init__(self, signal, slot):
        self.signal = signal
        self.slot = slot

    def __enter__(self):
        self.signal.disconnect(self.slot)
        return self

    def __exit__(self, *args):
        self.signal.connect(self.slot)
        

class LabelTree(QWidget):

    class SignalProxy(QObject):
        labelColorChanged = pyqtSignal(object)
        labelsChanged = pyqtSignal()
        resetLabels = pyqtSignal()
    
    def __init__(self, parent=None):
        self._sigprox = LabelTree.SignalProxy()
        self.label_color_changed = self._sigprox.labelColorChanged
        self.labels_changed = self._sigprox.labelsChanged
        self.reset_labels = self._sigprox.resetLabels

        self._block_signals = False
        QWidget.__init__(self, parent)
        self.setStyleSheet(label_tree_style)

        self.layout = QVBoxLayout()
        self.setLayout(self.layout)
        self.layout.setSpacing(0)
        self.layout.setContentsMargins(0, 0, 0, 0)

        self.label_level = None
        self.current_lut = None
        self.root_item = []
        self.root_acronym = []
        
        self.tree = QTreeWidget(self)
        self.layout.addWidget(self.tree)
        self.tree.header().setResizeMode(QHeaderView.ResizeToContents)
        self.tree.headerItem().setText(0, "id")
        self.tree.headerItem().setText(1, "name")
        self.tree.headerItem().setText(2, "color")
        self.labels_by_id = {}
        self.labels_by_acronym = {}
        self.checked = set()
        self.tree.itemChanged.connect(self.item_change)

        self.layout.addSpacing(10)
        self.reset_btn = QPushButton('Reset colors')
        self.reset_btn.setStyleSheet(reset_button_style)
        self.layout.addWidget(self.reset_btn)
        self.reset_btn.clicked.connect(self.reset_colors)
    
    def set_labels(self, label_data):
        self._block_signals = True
        try:
            if self.current_lut is not None:
                self.clear_labels()

            n_labels = len(label_data['index'])
            self.label_level = np.max([ind for ind in label_data['index'] if ind not in label_data['parent']])
            self.current_lut = np.zeros((self.label_level + 1, 4), 'i')
            for i in range(n_labels):
                label_id = label_data['index'][i]
                parent = label_data['parent'][i]
                color = label_data['color'][i]
                if label_id <= self.label_level:
                    self.current_lut[label_id] = np.array([color[0], color[1], color[2], 255])
                da_color = QColor(color[0], color[1], color[2]).name(QColor.HexRgb)
                da_color = da_color.split('#')[1]
                name = label_data['label'][i]
                acronym = label_data['abbrev'][i]
                if parent < 0:
                    parent = -1
                    self.root_acronym.append(acronym.encode())
                rec = (label_id, parent, name.encode(), acronym.encode(), da_color.encode())
                self.add_label(*rec)
            for i in range(len(self.root_acronym)):
                self.root_item.append(self.labels_by_acronym[self.root_acronym[i]]['item'])
        finally:
            self._block_signals = False
        self.labels_changed.emit()

    def add_label(self, label_id, parent, name, acronym, color):
        item = QTreeWidgetItem([acronym.decode(), name.decode(), ''])
        item.setFlags(item.flags() | Qt.ItemIsUserCheckable)
        item.setCheckState(0, Qt.Unchecked)
        if parent in self.labels_by_id:
            root = self.labels_by_id[parent]['item']
        else:
            root = self.tree.invisibleRootItem()
    
        root.addChild(item)
        btn = pg.ColorButton(color=pg.mkColor(color.decode()))
        btn.defaultColor = color.decode()
        btn.id = label_id
        self.tree.setItemWidget(item, 2, btn)
    
        self.labels_by_id[label_id] = {'item': item, 'btn': btn}
        item.id = label_id
        self.labels_by_acronym[acronym] = self.labels_by_id[label_id]
    
        btn.sigColorChanged.connect(self.item_color_changed)

    def clear_labels(self):
        root = self.tree.invisibleRootItem()
        for label_id in list(self.labels_by_id.keys()):
            da_item = self.labels_by_id[label_id]['item']
            (da_item.parent() or root).removeChild(da_item)

    def item_change(self, item, col):
        checked = item.checkState(0) == Qt.Checked
        with SignalBlock(self.tree.itemChanged, self.item_change):
            self.check_recursive(item, checked)
    
        if not self._block_signals:
            self.labels_changed.emit()

    def check_recursive(self, item, checked):
        if checked:
            self.checked.add(item.id)
            item.setCheckState(0, Qt.Checked)
        else:
            if item.id in self.checked:
                self.checked.remove(item.id)
            item.setCheckState(0, Qt.Unchecked)
    
        for i in range(item.childCount()):
            self.check_recursive(item.child(i), checked)

    def item_color_changed(self, btn):
        color = btn.color()
        self.set_label_color(btn.id, color)

    def set_label_color(self, label_id, color, recursive=True, emit=True):
        item = self.labels_by_id[label_id]['item']
        btn = self.labels_by_id[label_id]['btn']
        print(color)
        rgb_color = (color.red(), color.green(), color.blue(), color.alpha())
        self.current_lut[label_id] = np.array([rgb_color[0], rgb_color[1], rgb_color[2], rgb_color[3]])
        with SignalBlock(btn.sigColorChanged, self.item_color_changed):
            btn.setColor(color)
        if recursive:
            for i in range(item.childCount()):
                ch = item.child(i)
                self.set_label_color(ch.id, color, recursive=recursive, emit=False)
        if emit:
            self.label_color_changed.emit((label_id, self.current_lut[label_id]))
    
    def lookup_table(self):
        lut = np.zeros((self.label_level + 1, 4), dtype=np.ubyte)
        # print(lut.shape)
        for layer_id in self.checked:
            if layer_id >= lut.shape[0]:
                continue
            lut[layer_id] = self.labels_by_id[layer_id]['btn'].color(mode='byte')
        return lut

    def reset_colors(self):
        try:
            self.blockSignals(True)
            for k, v in self.labels_by_id.items():
                self.set_label_color(k, v['btn'].defaultColor, recursive=False, emit=False)
        finally:
            self.blockSignals(False)
            self.reset_labels.emit()
    
    def describe(self, label_id):
        if label_id == 0:
            return ''
        else:
            if label_id not in self.labels_by_id:
                return "Unknown label: %d" % label_id
        descr = []
        item = self.labels_by_id[label_id]['item']
        name = str(item.text(1))
        while item not in self.root_item:
            # self.labels_by_acronym[b'Brain']['item'], self.labels_by_acronym[b'SpC']['item'], self.labels_by_acronym[b'IE']['item']
            descr.insert(0, str(item.text(0)))
            item = item.parent()
        return '[%d]' % label_id + ' > '.join(descr) + "  :  " + name


import os
import sys
import numpy as np
from random import randint
from PyQt5.QtWidgets import *
import pyqtgraph as pg
from PyQt5.QtGui import QIcon, QPixmap, QColor
from PyQt5.QtCore import Qt, QSize
from .wtiles import *

btm_style = '''

QPushButton {
    border : None; 
    background: transparent;
    margin: 0px;
    padding-top: 0px;
    border-radius: 0px;
    min-width: 24px;
    min-height: 24px;
}
QPushButton:checked {
    background-color: #383838; 
    border: 1px solid #636363; 
}

QPushButton:pressed {
    background-color: #383838; 
    border: 1px solid #636363; 
}

QPushButton:hover {
    background-color: #383838; 
    border: 1px solid #636363; 
}

'''


eye_button_style = '''
    border-left: None;
    border-right: 1px solid gray;
    border-top: None;
    border-bottom: None;
    border-radius: 0px;
    background: transparent;
'''


text_btn_style = '''
QPushButton {
    background: transparent;
    border: None;
    border-radius:0px;
    text-align:left; 
    padding-left: 5px; 
    padding-right: 0px;
    padding-top: 0px;
    padding-bottom: 0px;
    color:rgb(240, 240, 240);
    margin: 0px;
    height: 40px;
    width: 220px;
}


QPushButton:disabled {
    color: rgb(190, 190, 190);
}
'''


line_edit_style = '''
QLineEdit {
    background: transparent;
    border: 1px solid green;
    border-radius:0px;
    text-align:left; 
    color: white; 
    padding-left: 5px;    
    margin: 0px;
    height: 40px;
    width: 220px; 
}
'''


class CompareWindow(QDialog):
    def __init__(self, obj_names, obj_data):
        super().__init__()

        self.setWindowTitle("Compare Information Window")

        n_object = len(obj_names)
        all_label_names = obj_data[0]['label_name'].copy()
        all_label_acronym = obj_data[0]['label_acronym'].copy()
        all_label_color = obj_data[0]['label_color'].copy()
        all_label_channels = obj_data[0]['region_channels'].tolist()

        for i in range(1, n_object):
            check_label = obj_data[i]['label_name']
            for j in range(len(check_label)):
                if check_label[j] in all_label_names:
                    da_ind = np.where(np.ravel(all_label_names) == check_label[j])[0][0]
                    all_label_channels[da_ind] = all_label_channels[da_ind] + obj_data[i]['region_channels'][j]
                else:
                    all_label_names.append(check_label[j])
                    all_label_acronym.append(obj_data[i]['label_acronym'][j])
                    all_label_color.append(obj_data[i]['label_color'][j])
                    all_label_channels.append(obj_data[i]['region_channels'][j])

        layout = QVBoxLayout()
        full_name = obj_names[0]
        for i in range(1, n_object):
            full_name = full_name + ', '
            full_name = full_name + obj_names[i]

        self.label = QLabel('Compare {}'.format(full_name))
        color = QColor(128, 128, 128, 128)
        label_style = 'QLabel {background-color: ' + color.name() + '; font-size: 20px}'
        self.label.setStyleSheet(label_style)

        sec_group = QGroupBox()
        slayout = QGridLayout(sec_group)
        lb1 = QLabel('Brain Region')
        lb2 = QLabel('Acronym')
        lb3 = QLabel('Channels')
        lb4 = QLabel('Color')
        slayout.addWidget(lb1, 0, 0, 1, 1)
        slayout.addWidget(lb2, 0, 1, 1, 1)
        slayout.addWidget(lb3, 0, 2, 1, 1)
        slayout.addWidget(lb4, 0, 3, 1, 1)

        channels = np.ravel(all_label_channels).astype(int)

        for i in range(len(all_label_names)):
            slayout.addWidget(QLabel(all_label_names[i]), i + 1, 0, 1, 1)
            slayout.addWidget(QLabel(all_label_acronym[i]), i + 1, 1, 1, 1)
            slayout.addWidget(QLabel(str(channels[i])), i + 1, 2, 1, 1)
            clb = QLabel()
            da_color = QColor(all_label_color[i][0], all_label_color[i][1], all_label_color[i][2], 255).name()
            clb.setStyleSheet('QLabel {background-color: ' + da_color + '; width: 20px; height: 20px}')
            slayout.addWidget(clb, i + 1, 3, 1, 1)

        # make plot data
        plot_frame = QFrame()
        plot_frame.setMaximumWidth(300)
        plot_frame.setMinimumWidth(300)
        view_layout = QHBoxLayout(plot_frame)
        view_layout.setSpacing(0)
        view_layout.setContentsMargins(0, 0, 0, 0)

        w = pg.GraphicsLayoutWidget()
        view = w.addViewBox()
        view.setAspectLocked()

        view_layout.addWidget(w)

        resize_factor = 5
        max_num_channel = 0
        for i in range(n_object):
            da_sum = np.sum(obj_data[i]['block_count'][0])
            if da_sum > max_num_channel:
                max_num_channel = da_sum

        for i in range(n_object):
            count1 = obj_data[i]['block_count'][0]
            plot_data1 = np.cumsum(count1)
            plot_data1 = plot_data1 / max_num_channel * resize_factor
            plot_colors1 = np.asarray(obj_data[i]['merged_color'][0])

            count2 = obj_data[i]['block_count'][1]
            plot_data2 = np.cumsum(count2)
            plot_data2 = plot_data2 / max_num_channel * resize_factor
            plot_colors2 = np.asarray(obj_data[i]['merged_color'][1])

            # draw tips
            da_tip_loc = np.array([[i - 0.3, 0], [i, - 0.7], [i + 0.3, 0]])
            tips = pg.PlotDataItem(da_tip_loc, connect='all', fillLevel=0, fillBrush=pg.mkBrush(color=(128, 128, 128)))
            view.addItem(tips)

            # create bar chart
            bg_list1 = []
            for j in np.arange(len(plot_data1))[::-1]:
                bg = pg.BarGraphItem(x=[i - 0.15], height=plot_data1[j], width=0.3,
                                     brush=QColor(plot_colors1[j][0], plot_colors1[j][1], plot_colors1[j][2], 255))
                bg_list1.append(bg)
                view.addItem(bg_list1[-1])

            bg_list2 = []
            for j in np.arange(len(plot_data2))[::-1]:
                bg = pg.BarGraphItem(x=[i + 0.15], height=plot_data2[j], width=0.3,
                                     brush=QColor(plot_colors2[j][0], plot_colors2[j][1], plot_colors2[j][2], 255))
                bg_list2.append(bg)
                view.addItem(bg_list2[-1])

        channel_info_frame = QFrame()
        channel_info_layout = QHBoxLayout(channel_info_frame)
        channel_info_layout.setContentsMargins(0, 0, 0, 0)
        channel_info_layout.setSpacing(10)
        channel_info_layout.addWidget(plot_frame)
        channel_info_layout.addWidget(sec_group)

        # ok button, used to close window
        ok_btn = QDialogButtonBox(QDialogButtonBox.Ok)
        ok_btn.accepted.connect(self.accept)

        # add widget to layout
        layout.addWidget(self.label)
        layout.addWidget(channel_info_frame)
        layout.addWidget(ok_btn)
        self.setLayout(layout)

    def accept(self) -> None:
        self.close()


class CellsInfoWindow(QDialog):
    def __init__(self, name, data):
        super().__init__()

        self.setWindowTitle("Cell Information Window")

        layout = QVBoxLayout()
        self.label = QLabel(name)
        color = QColor(data['vis_color'][0], data['vis_color'][1], data['vis_color'][2], data['vis_color'][3])
        label_style = 'QLabel {background-color: ' + color.name() + '; font-size: 20px}'
        self.label.setStyleSheet(label_style)

        sec_group = QGroupBox('Total Count: {}'.format(len(data['data'])))
        slayout = QGridLayout(sec_group)
        lb1 = QLabel('Brain Region')
        lb2 = QLabel('Acronym')
        lb3 = QLabel('Color')
        lb4 = QLabel('Count')
        slayout.addWidget(lb1, 0, 0, 1, 1)
        slayout.addWidget(lb2, 0, 1, 1, 1)
        slayout.addWidget(lb3, 0, 2, 1, 1)
        slayout.addWidget(lb4, 0, 3, 1, 1)

        for i in range(len(data['label_name'])):
            slayout.addWidget(QLabel(data['label_name'][i]), i + 1, 0, 1, 1)
            slayout.addWidget(QLabel(data['label_acronym'][i]), i + 1, 1, 1, 1)
            clb = QLabel()
            da_color = QColor(data['label_color'][i][0], data['label_color'][i][1], data['label_color'][i][2],
                              255).name()
            clb.setStyleSheet('QLabel {background-color: ' + da_color + '; width: 20px; height: 20px}')
            slayout.addWidget(clb, i + 1, 2, 1, 1)
            slayout.addWidget(QLabel(str(data['region_count'][i])), i + 1, 3, 1, 1)

        # ok button, used to close window
        ok_btn = QDialogButtonBox(QDialogButtonBox.Ok)
        ok_btn.accepted.connect(self.accept)

        # add widget to layout
        layout.addWidget(self.label)
        layout.addWidget(sec_group)
        layout.addWidget(ok_btn)
        self.setLayout(layout)

    def accept(self) -> None:
        self.close()

    def set_probe_color(self, color):
        self.label.setStyleSheet('QLabel {background-color: ' + color + ';}')


class VirusInfoWindow(QDialog):
    def __init__(self, name, data):
        super().__init__()

        self.setWindowTitle("Virus Information Window")

        layout = QVBoxLayout()
        self.label = QLabel(name)
        color = QColor(data['vis_color'][0], data['vis_color'][1], data['vis_color'][2], data['vis_color'][3])
        label_style = 'QLabel {background-color: ' + color.name() + '; font-size: 20px}'
        self.label.setStyleSheet(label_style)

        sec_group = QGroupBox()
        slayout = QGridLayout(sec_group)
        lb1 = QLabel('Brain Region')
        lb2 = QLabel('Acronym')
        lb3 = QLabel('Color')
        slayout.addWidget(lb1, 0, 0, 1, 1)
        slayout.addWidget(lb2, 0, 1, 1, 1)
        slayout.addWidget(lb3, 0, 2, 1, 1)

        for i in range(len(data['label_name'])):
            slayout.addWidget(QLabel(data['label_name'][i]), i + 1, 0, 1, 1)
            slayout.addWidget(QLabel(data['label_acronym'][i]), i + 1, 1, 1, 1)
            clb = QLabel()
            da_color = QColor(data['label_color'][i][0], data['label_color'][i][1], data['label_color'][i][2],
                              255).name()
            clb.setStyleSheet('QLabel {background-color: ' + da_color + '; width: 20px; height: 20px}')
            slayout.addWidget(clb, i + 1, 2, 1, 1)

        # ok button, used to close window
        ok_btn = QDialogButtonBox(QDialogButtonBox.Ok)
        ok_btn.accepted.connect(self.accept)

        # add widget to layout
        layout.addWidget(self.label)
        layout.addWidget(sec_group)
        layout.addWidget(ok_btn)
        self.setLayout(layout)

    def accept(self) -> None:
        self.close()

    def set_probe_color(self, color):
        self.label.setStyleSheet('QLabel {background-color: ' + color + ';}')


class ProbeInfoWindow(QDialog):
    def __init__(self, name, data):
        super().__init__()

        self.setWindowTitle("Probe Information Window")

        self.label = QLabel(name)
        # self.label = QLabel("Probe % d " % group_id)
        color = QColor(data['vis_color'][0], data['vis_color'][1], data['vis_color'][2], data['vis_color'][3])
        label_style = 'QLabel {background-color: ' + color.name() + '; font-size: 20px}'
        self.label.setStyleSheet(label_style)

        ap_angle_label = QLabel('AP Angle : ')
        ap_angle_label.setAlignment(Qt.AlignCenter)
        ml_angle_label = QLabel('ML Angle : ')
        ml_angle_label.setAlignment(Qt.AlignCenter)
        probe_length_label = QLabel('Probe Length : ')
        probe_length_label.setAlignment(Qt.AlignCenter)
        dv_label = QLabel('DV : ')
        dv_label.setAlignment(Qt.AlignCenter)
        insertion_coords_label = QLabel('Insertion coordinates : ')
        insertion_coords_label.setAlignment(Qt.AlignCenter)
        insertion_voxels_label = QLabel('Insertion voxels : ')
        insertion_voxels_label.setAlignment(Qt.AlignCenter)

        terminus_coords_label = QLabel('Terminus coordinates : ')
        terminus_coords_label.setAlignment(Qt.AlignCenter)
        terminus_voxels_label = QLabel('Terminus voxels : ')
        terminus_voxels_label.setAlignment(Qt.AlignCenter)

        ap_angle_value = QLabel('{} \u00B0'.format(np.round(data['ap_angle'], 2)))
        ml_angle_value = QLabel('{} \u00B0'.format(np.round(data['ml_angle'], 2)))
        probe_length = QLabel('{} \u03BCm'.format(np.round(data['probe_length'], 2)))
        dv = QLabel('{} \u03BCm'.format(np.round(data['dv'], 2)))
        ic_val = np.round(data['insertion_coords'], 2)
        insertion_coords = QLabel('ML: {}\u03BCm,  AP: {}\u03BCm'.format(ic_val[0], ic_val[1]))
        iv_val = data['insertion_vox'].astype(int)
        insertion_vox = QLabel('({} {} {})'.format(iv_val[0], iv_val[1], iv_val[2]))

        tc_val = np.round(data['terminus_coords'], 2)
        terminus_coords = QLabel('ML: {}\u03BCm,  AP: {}\u03BCm'.format(tc_val[0], tc_val[1]))
        tv_val = data['terminus_vox']
        terminus_vox = QLabel('({} {} {})'.format(tv_val[0], tv_val[1], tv_val[2]))

        coords_info_group = QGroupBox()
        coords_info_layout = QGridLayout(coords_info_group)
        coords_info_layout.addWidget(ap_angle_label, 0, 0, 1, 1)
        coords_info_layout.addWidget(ap_angle_value, 0, 1, 1, 1)
        coords_info_layout.addWidget(ml_angle_label, 0, 2, 1, 1)
        coords_info_layout.addWidget(ml_angle_value, 0, 3, 1, 1)

        coords_info_layout.addWidget(probe_length_label, 1, 0, 1, 1)
        coords_info_layout.addWidget(probe_length, 1, 1, 1, 1)
        coords_info_layout.addWidget(dv_label, 1, 2, 1, 1)
        coords_info_layout.addWidget(dv, 1, 3, 1, 1)

        coords_info_layout.addWidget(insertion_coords_label, 2, 0, 1, 1)
        coords_info_layout.addWidget(insertion_coords, 2, 1, 1, 1)
        coords_info_layout.addWidget(terminus_coords_label, 2, 2, 1, 1)
        coords_info_layout.addWidget(terminus_coords, 2, 3, 1, 1)

        coords_info_layout.addWidget(insertion_voxels_label, 3, 0, 1, 1)
        coords_info_layout.addWidget(insertion_vox, 3, 1, 1, 1)
        coords_info_layout.addWidget(terminus_voxels_label, 3, 2, 1, 1)
        coords_info_layout.addWidget(terminus_vox, 3, 3, 1, 1)

        sec_group = QGroupBox()
        slayout = QGridLayout(sec_group)
        lb1 = QLabel('Brain Region')
        lb2 = QLabel('Acronym')
        lb3 = QLabel('Channels')
        lb4 = QLabel('Color')
        slayout.addWidget(lb1, 0, 0, 1, 1)
        slayout.addWidget(lb2, 0, 1, 1, 1)
        slayout.addWidget(lb3, 0, 2, 1, 1)
        slayout.addWidget(lb4, 0, 3, 1, 1)

        channels = np.ravel(data['region_channels'].astype(int)).astype(str)

        for i in range(len(data['label_name'])):
            slayout.addWidget(QLabel(data['label_name'][i]), i + 1, 0, 1, 1)
            slayout.addWidget(QLabel(data['label_acronym'][i]), i + 1, 1, 1, 1)
            slayout.addWidget(QLabel(channels[i]), i + 1, 2, 1, 1)
            clb = QLabel()
            da_color = QColor(data['label_color'][i][0], data['label_color'][i][1], data['label_color'][i][2], 255).name()
            clb.setStyleSheet('QLabel {background-color: ' + da_color + '; width: 20px; height: 20px}')
            slayout.addWidget(clb, i + 1, 3, 1, 1)

        # make plot data
        resize_factor = 5
        count1 = data['block_count'][0]
        text_data1 = count1 / np.sum(count1) * resize_factor
        plot_data1 = np.cumsum(count1)
        plot_data1 = plot_data1 / plot_data1[-1] * resize_factor
        # plot_data1 = plot_data1[::-1]
        plot_colors1 = np.asarray(data['merged_color'][0])

        count2 = data['block_count'][1]
        text_data2 = count2 / np.sum(count2) * resize_factor
        plot_data2 = np.cumsum(count2)
        plot_data2 = plot_data2 / plot_data2[-1] * resize_factor
        # plot_data2 = plot_data2[::-1]
        plot_colors2 = np.asarray(data['merged_color'][1])
        # self.label = label[::-1]


        plot_frame = QFrame()
        plot_frame.setMaximumWidth(300)
        plot_frame.setMinimumWidth(300)
        view_layout = QHBoxLayout(plot_frame)
        view_layout.setSpacing(0)
        view_layout.setContentsMargins(0, 0, 0, 0)

        w = pg.GraphicsLayoutWidget()
        view = w.addViewBox()
        view.setAspectLocked()
        # view.invertY(True)

        # draw tips
        da_tip_loc = np.array([[0.7, 0], [1, - 0.7], [1.3, 0]])
        tips = pg.PlotDataItem(da_tip_loc, connect='all', fillLevel=0, fillBrush=pg.mkBrush(color=(128, 128, 128)))
        view.addItem(tips)

        # create bar chart
        bg_list1 = []
        for i in np.arange(len(plot_data1))[::-1]:
            bg = pg.BarGraphItem(x=[1-0.15], height=plot_data1[i], width=0.3,
                                 brush=QColor(plot_colors1[i][0], plot_colors1[i][1], plot_colors1[i][2], 255))
            bg_list1.append(bg)
            view.addItem(bg_list1[-1])

        text_list1 = [pg.TextItem(str(count1[0]))]
        text_list1[-1].setPos(0.6, text_data1[0] * 0.5)
        text_list1[-1].setAnchor((0.5, 0.5))
        view.addItem(text_list1[-1])
        for i in range(1, len(plot_data1)):
            tt = pg.TextItem(str(count1[i]))
            if np.mod(i, 2) == 0:
                tt.setPos(0.6, plot_data1[i - 1] + text_data1[i] * 0.5)
            else:
                tt.setPos(0.5, plot_data1[i - 1] + text_data1[i] * 0.5)
            tt.setAnchor((0.5, 0.5))
            text_list1.append(tt)
            view.addItem(text_list1[i])

        bg_list2 = []
        for i in np.arange(len(plot_data2))[::-1]:
            bg = pg.BarGraphItem(x=[1+0.15], height=plot_data2[i], width=0.3,
                                 brush=QColor(plot_colors2[i][0], plot_colors2[i][1], plot_colors2[i][2], 255))
            bg_list2.append(bg)
            view.addItem(bg_list2[-1])

        text_list2 = [pg.TextItem(str(count2[0]))]
        text_list2[-1].setPos(1.4, text_data2[0] * 0.5)
        text_list2[-1].setAnchor((0.5, 0.5))
        view.addItem(text_list2[-1])
        for i in range(1, len(plot_data2)):
            tt = pg.TextItem(str(count2[i]))
            if np.mod(i, 2) == 0:
                tt.setPos(1.4, plot_data2[i - 1] + text_data2[i] * 0.5)
            else:
                tt.setPos(1.5, plot_data2[i - 1] + text_data2[i] * 0.5)
            tt.setAnchor((0.5, 0.5))
            text_list2.append(tt)
            view.addItem(text_list2[i])

        view_layout.addWidget(w)

        channel_info_frame = QFrame()
        channel_info_layout = QHBoxLayout(channel_info_frame)
        channel_info_layout.setContentsMargins(0, 0, 0, 0)
        channel_info_layout.setSpacing(10)
        channel_info_layout.addWidget(plot_frame)
        channel_info_layout.addWidget(sec_group)

        # ok button, used to close window
        ok_btn = QDialogButtonBox(QDialogButtonBox.Ok)
        ok_btn.accepted.connect(self.accept)

        # add widget to layout
        layout = QVBoxLayout()
        layout.addWidget(self.label)
        layout.addWidget(coords_info_group)
        layout.addSpacing(10)
        layout.addWidget(channel_info_frame)
        layout.addWidget(ok_btn)
        self.setLayout(layout)

    def accept(self) -> None:
        self.close()

    def set_probe_color(self, color):
        self.label.setStyleSheet('QLabel {background-color: ' + color + ';}')


class SinglePiece(QWidget):
    sig_clicked = pyqtSignal(object)
    sig_name_changed = pyqtSignal(object)

    def __init__(self, parent=None, index=0, obj_name='', obj_type='probe piece', object_icon=None):
        QWidget.__init__(self, parent=parent)

        self.inactive_style = 'QFrame{background-color:rgb(83, 83, 83); border: 1px solid rgb(128, 128, 128);}'
        self.active_style = 'QFrame{background-color:rgb(107, 107, 107); border: 1px solid rgb(128, 128, 128);}'

        self.id = index
        self.object_name = obj_name
        self.object_type = obj_type
        self.active = True

        self.tbnail = QPushButton()
        self.tbnail.setFixedSize(QSize(40, 40))
        self.tbnail.setStyleSheet(eye_button_style)
        self.tbnail.setIcon(object_icon)
        self.tbnail.setIconSize(QSize(20, 20))
        self.tbnail.clicked.connect(self.on_click)

        self.text_btn = QDoubleButton()
        self.text_btn.setStyleSheet(text_btn_style)
        self.text_btn.setFixedSize(QSize(240, 40))
        self.text_btn.left_clicked.connect(self.on_click)
        self.text_btn.double_clicked.connect(self.on_doubleclick)

        self.l_line_edit = QLineEdit()
        self.l_line_edit.setStyleSheet(line_edit_style)
        self.l_line_edit.setFixedWidth(240)
        self.l_line_edit.editingFinished.connect(self.enter_pressed)
        self.l_line_edit.setVisible(False)

        self.inner_frame = QFrame()
        self.inner_frame.setStyleSheet(self.active_style)
        self.inner_layout = QHBoxLayout(self.inner_frame)
        self.inner_layout.setContentsMargins(0, 0, 0, 0)
        self.inner_layout.setSpacing(0)
        self.inner_layout.setAlignment(Qt.AlignVCenter)
        self.inner_layout.addWidget(self.tbnail)
        self.inner_layout.addSpacing(5)
        self.inner_layout.addWidget(self.text_btn)
        self.inner_layout.addWidget(self.l_line_edit)
        self.inner_layout.addStretch()

        outer_layout = QHBoxLayout()
        outer_layout.setContentsMargins(0, 0, 0, 0)
        outer_layout.setSpacing(0)
        outer_layout.setAlignment(Qt.AlignVCenter)
        outer_layout.addWidget(self.inner_frame)

        self.setLayout(outer_layout)
        self.setFixedHeight(40)
        # self.show()

    @pyqtSlot()
    def on_click(self):
        self.set_checked(True)
        self.sig_clicked.emit(self.id)

    @pyqtSlot()
    def on_doubleclick(self):
        self.l_line_edit.setText(self.text_btn.text())
        self.l_line_edit.setVisible(True)
        self.text_btn.setVisible(False)
        self.l_line_edit.setFocus(True)
        self.set_checked(True)
        self.sig_clicked.emit(self.id)

    @pyqtSlot()
    def enter_pressed(self):
        da_text = self.l_line_edit.text()
        if '-' not in da_text or 'piece' not in da_text:
            return
        self.l_line_edit.setVisible(False)
        self.text_btn.setText(self.l_line_edit.text())
        self.object_name = self.l_line_edit.text()
        self.text_btn.setVisible(True)
        self.sig_name_changed.emit((self.id, self.object_name))

    def is_checked(self):
        return self.active

    def set_checked(self, check):
        self.active = check
        if not self.active:
            self.inner_frame.setStyleSheet(self.inactive_style)
        else:
            self.inner_frame.setStyleSheet(self.active_style)


class RegisteredObject(QWidget):
    sig_object_color_changed = pyqtSignal(object)
    eye_clicked = pyqtSignal(object)
    sig_clicked = pyqtSignal(object)
    sig_link = pyqtSignal(object)

    def __init__(self, parent=None, obj_id=0, obj_name='', obj_type='merged probe', object_icon=None):
        QWidget.__init__(self, parent=parent)

        self.inactive_style = 'QFrame{background-color:rgb(83, 83, 83); border: 1px solid rgb(128, 128, 128);}'
        self.active_style = 'QFrame{background-color:rgb(107, 107, 107); border: 1px solid rgb(128, 128, 128);}'

        self.color = QColor(randint(0, 255), randint(0, 255), randint(0, 255), 255)
        self.icon_back = 'border:1px solid black; background-color: {}'.format(self.color.name())

        self.id = obj_id
        self.object_type = obj_type
        self.object_name = obj_name
        self.active = True
        self.vis = True

        self.eye_button = QPushButton()
        self.eye_button.setFixedSize(QSize(40, 40))
        self.eye_button.setStyleSheet(eye_button_style)
        self.eye_button.setCheckable(True)
        eye_icon = QIcon()
        eye_icon.addPixmap(QPixmap("icons/layers/eye_on.png"), QIcon.Normal, QIcon.Off)
        eye_icon.addPixmap(QPixmap("icons/layers/eye_off.png"), QIcon.Normal, QIcon.On)
        self.eye_button.setIcon(eye_icon)
        self.eye_button.setIconSize(QSize(20, 20))
        self.eye_button.clicked.connect(self.eye_on_click)

        self.tbnail = QPushButton()
        self.tbnail.setStyleSheet(self.icon_back)
        self.tbnail.setIcon(object_icon)
        self.tbnail.setIconSize(QSize(20, 20))
        self.tbnail.clicked.connect(self.change_object_color)

        self.text_btn = QPushButton(self.object_name)
        self.text_btn.setStyleSheet(text_btn_style)
        self.text_btn.setFixedSize(QSize(150, 40))
        # self.text_btn.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Ignored)
        self.text_btn.clicked.connect(self.on_click)

        self.link_button = QPushButton()
        self.link_button.setFixedSize(QSize(25, 40))
        self.link_button.setCheckable(True)
        link_icon = QIcon()
        link_icon.addPixmap(QPixmap("icons/sidebar/link_off.svg"), QIcon.Normal, QIcon.Off)
        link_icon.addPixmap(QPixmap("icons/sidebar/link.svg"), QIcon.Normal, QIcon.On)
        self.link_button.setIcon(link_icon)
        self.link_button.setIconSize(QSize(20, 20))
        self.link_button.clicked.connect(self.on_linked)

        self.inner_frame = QFrame()
        self.inner_frame.setStyleSheet(self.active_style)
        self.inner_layout = QHBoxLayout(self.inner_frame)
        self.inner_layout.setContentsMargins(0, 0, 0, 0)
        self.inner_layout.setSpacing(0)
        self.inner_layout.setAlignment(Qt.AlignVCenter)
        self.inner_layout.addWidget(self.eye_button)
        self.inner_layout.addSpacing(5)
        self.inner_layout.addWidget(self.tbnail)
        self.inner_layout.addSpacing(5)
        self.inner_layout.addWidget(self.text_btn)
        self.inner_layout.addWidget(self.link_button)
        self.inner_layout.addStretch()

        outer_layout = QHBoxLayout()
        outer_layout.setContentsMargins(0, 0, 0, 0)
        outer_layout.setSpacing(0)
        outer_layout.setAlignment(Qt.AlignVCenter)
        outer_layout.addWidget(self.inner_frame)

        self.setLayout(outer_layout)
        self.setFixedHeight(40)

    def set_icon_style(self, color):
        self.color = color
        self.icon_back = 'border:1px solid black; background-color: {}'.format(color.name())
        self.tbnail.setStyleSheet(self.icon_back)

    def change_object_color(self):
        self.sig_object_color_changed.emit(self.id)

    def eye_on_click(self):
        if self.eye_button.isChecked():
            self.vis = False
        else:
            self.vis = True
        self.set_checked(True)
        self.tbnail.setEnabled(self.vis)
        self.text_btn.setEnabled(self.vis)
        self.eye_clicked.emit((self.id, self.vis))

    def set_checked(self, check):
        self.active = check
        if not self.active:
            self.inner_frame.setStyleSheet(self.inactive_style)
        else:
            self.inner_frame.setStyleSheet(self.active_style)

    def on_click(self):
        print("Click")
        self.set_checked(True)
        self.sig_clicked.emit(self.id)

    def is_checked(self):
        return self.active

    def on_linked(self):
        self.sig_link.emit(self.id)


class ObjectControl(QObject):
    """
    only pieces' name can be changed, the merged can not, so far
    """

    class SignalProxy(QObject):
        sigOpacityChanged = pyqtSignal(object)
        sigVisChanged = pyqtSignal(object)
        sigDeleteObject = pyqtSignal(object)
        sigColorChanged = pyqtSignal(object)
        sigSizeChanged = pyqtSignal(object)
        sigBlendModeChanged = pyqtSignal(object)

    def __init__(self, parent=None):

        self._sigprox = ObjectControl.SignalProxy()
        self.sig_opacity_changed = self._sigprox.sigOpacityChanged
        self.sig_visible_changed = self._sigprox.sigVisChanged
        self.sig_delete_object = self._sigprox.sigDeleteObject
        self.sig_color_changed = self._sigprox.sigColorChanged
        self.sig_size_changed = self._sigprox.sigSizeChanged
        self.sig_blend_mode_changed = self._sigprox.sigBlendModeChanged

        QObject.__init__(self)

        self.default_size_val = 2
        self.default_opacity_val = 100

        self.valid_obj_type = ['probe piece', 'merged probe',
                               'cells piece', 'merged cells',
                               'virus piece', 'merged virus',
                               'contour piece', 'merged contour',
                               'drawing piece', 'merged drawing']

        self.current_obj_index = None

        self.obj_count = 0
        self.linked_indexes = []

        self.obj_list = []  # widgets
        self.obj_id = []  # identity
        self.obj_name = []  # names, initially the same as type, can be changed freely ???
        self.obj_type = []  # type
        self.obj_data = []  # data
        self.obj_size = []  # size
        self.obj_opacity = []
        self.obj_comp_mode = []

        self.probe_icon = QIcon('icons/sidebar/probe.svg')
        self.virus_icon = QIcon('icons/sidebar/virus.svg')
        self.drawing_icon = QIcon('icons/toolbar/pencil.svg')
        self.cell_icon = QIcon('icons/toolbar/location.svg')
        self.contour_icon = QIcon('icons/sidebar/contour.svg')
        self.compare_icon = QIcon('icons/sidebar/compare.svg')

        combo_label = QLabel('Composition:')
        combo_label.setFixedWidth(80)
        self.obj_blend_combo = QComboBox()
        self.obj_blend_combo.setEditable(False)
        combo_value = ['opaque', 'translucent', 'additive']
        self.obj_blend_combo.addItems(combo_value)
        self.obj_blend_combo.setCurrentText('opaque')
        self.obj_blend_combo.currentTextChanged.connect(self.blend_mode_changed)
        combo_wrap = QFrame()
        combo_wrap_layout = QHBoxLayout(combo_wrap)
        combo_wrap_layout.setContentsMargins(0, 0, 0, 0)
        combo_wrap_layout.setSpacing(5)
        combo_wrap_layout.addWidget(combo_label)
        combo_wrap_layout.addWidget(self.obj_blend_combo)

        obj_opacity_label = QLabel('Opacity:')
        obj_opacity_label.setFixedWidth(80)
        self.obj_opacity_slider = QSlider(Qt.Horizontal)
        self.obj_opacity_slider.setMaximum(100)
        self.obj_opacity_slider.setMinimum(0)
        self.obj_opacity_slider.setValue(100)
        self.obj_opacity_slider.valueChanged.connect(self.change_opacity_label_value)
        self.obj_opacity_slider.sliderMoved.connect(self.send_opacity_changed_signal)
        self.obj_opacity_val_label = QLabel('100%')
        self.obj_opacity_val_label.setFixedWidth(40)
        opacity_wrap = QFrame()
        opacity_wrap_layout = QHBoxLayout(opacity_wrap)
        opacity_wrap_layout.setContentsMargins(0, 0, 0, 0)
        opacity_wrap_layout.setSpacing(5)
        opacity_wrap_layout.addWidget(obj_opacity_label)
        opacity_wrap_layout.addWidget(self.obj_opacity_slider)
        opacity_wrap_layout.addWidget(self.obj_opacity_val_label)

        obj_size_label = QLabel('Size/Width: ')
        obj_size_label.setFixedWidth(80)
        self.obj_size_slider = QSlider(Qt.Horizontal)
        self.obj_size_slider.setValue(2)
        self.obj_size_slider.setMinimum(1)
        self.obj_size_slider.setMaximum(10)
        self.obj_size_slider.valueChanged.connect(self.change_size_label_value)
        self.obj_size_slider.sliderMoved.connect(self.send_size_changed_signal)
        self.obj_size_val_label = QLabel('2')
        self.obj_size_val_label.setAlignment(Qt.AlignCenter)
        self.obj_size_val_label.setFixedWidth(40)
        size_wrap = QFrame()
        size_wrap_layout = QHBoxLayout(size_wrap)
        size_wrap_layout.setContentsMargins(0, 0, 0, 0)
        size_wrap_layout.setSpacing(5)
        size_wrap_layout.addWidget(obj_size_label)
        size_wrap_layout.addWidget(self.obj_size_slider)
        size_wrap_layout.addWidget(self.obj_size_val_label)

        top_frame = QFrame()
        top_layout = QVBoxLayout(top_frame)
        top_layout.setContentsMargins(0, 0, 0, 0)
        top_layout.setSpacing(0)
        top_layout.addWidget(combo_wrap)
        top_layout.addSpacing(10)
        top_layout.addWidget(opacity_wrap)
        top_layout.addSpacing(10)
        top_layout.addWidget(size_wrap)
        top_layout.addSpacing(10)

        self.layer_frame = QFrame()
        self.layer_frame.setStyleSheet('background: transparent; border: 0px;')
        self.layer_frame.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.layer_layout = QBoxLayout(QBoxLayout.BottomToTop, self.layer_frame)
        self.layer_layout.setAlignment(Qt.AlignBottom)
        self.layer_layout.setContentsMargins(0, 0, 0, 0)
        self.layer_layout.setSpacing(5)

        self.layer_scroll = QScrollArea()
        self.layer_scroll.setStyleSheet('background: transparent; border: 0px;')
        self.layer_scroll.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.layer_scroll.setWidget(self.layer_frame)
        self.layer_scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self.layer_scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.layer_scroll.setWidgetResizable(True)

        mid_frame = QFrame()
        mid_frame.setStyleSheet('background: transparent; border: 1px solid rgb(128, 128, 128);')
        mid_frame.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        mid_layout = QGridLayout(mid_frame)
        mid_layout.setContentsMargins(0, 0, 0, 0)
        mid_layout.setSpacing(0)
        mid_layout.setAlignment(Qt.AlignBottom)
        mid_layout.addWidget(self.layer_scroll, 0, 0, 1, 1)

        self.outer_frame = QFrame()
        self.outer_frame.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        outer_layout = QVBoxLayout(self.outer_frame)
        outer_layout.setSpacing(0)
        outer_layout.addWidget(top_frame)
        outer_layout.addWidget(mid_frame)

        # bottom buttons
        self.compare_btn = QPushButton()
        self.compare_btn.setFixedSize(24, 24)
        self.compare_btn.setStyleSheet(btm_style)
        self.compare_btn.setIcon(QIcon('icons/sidebar/compare.svg'))
        self.compare_btn.setIconSize(QSize(20, 20))

        self.merge_probe_btn = QPushButton()
        self.merge_probe_btn.setFixedSize(24, 24)
        self.merge_probe_btn.setStyleSheet(btm_style)
        self.merge_probe_btn.setIcon(QIcon('icons/sidebar/probe.svg'))
        self.merge_probe_btn.setIconSize(QSize(20, 20))

        self.merge_drawing_btn = QPushButton()
        self.merge_drawing_btn.setFixedSize(24, 24)
        self.merge_drawing_btn.setStyleSheet(btm_style)
        self.merge_drawing_btn.setIcon(QIcon('icons/toolbar/pencil.svg'))
        self.merge_drawing_btn.setIconSize(QSize(20, 20))

        self.merge_cell_btn = QPushButton()
        self.merge_cell_btn.setFixedSize(24, 24)
        self.merge_cell_btn.setStyleSheet(btm_style)
        self.merge_cell_btn.setIcon(QIcon('icons/toolbar/location.svg'))
        self.merge_cell_btn.setIconSize(QSize(20, 20))

        self.merge_virus_btn = QPushButton()
        self.merge_virus_btn.setFixedSize(24, 24)
        self.merge_virus_btn.setStyleSheet(btm_style)
        self.merge_virus_btn.setIcon(QIcon('icons/sidebar/virus.svg'))
        self.merge_virus_btn.setIconSize(QSize(20, 20))

        self.merge_contour_btn = QPushButton()
        self.merge_contour_btn.setFixedSize(24, 24)
        self.merge_contour_btn.setStyleSheet(btm_style)
        self.merge_contour_btn.setIcon(QIcon('icons/sidebar/contour.svg'))
        self.merge_contour_btn.setIconSize(QSize(20, 20))

        self.add_object_btn = QPushButton()
        self.add_object_btn.setFixedSize(24, 24)
        self.add_object_btn.setStyleSheet(btm_style)
        self.add_object_btn.setIcon(QIcon('icons/sidebar/add.png'))
        self.add_object_btn.setIconSize(QSize(20, 20))

        self.delete_object_btn = QPushButton()
        self.delete_object_btn.setFixedSize(24, 24)
        self.delete_object_btn.setStyleSheet(btm_style)
        self.delete_object_btn.setIcon(QIcon('icons/sidebar/trash.png'))
        self.delete_object_btn.setIconSize(QSize(20, 20))
        self.delete_object_btn.clicked.connect(self.delete_object_btn_clicked)

    def blend_mode_changed(self):
        blend_mode = self.obj_blend_combo.currentText()
        if self.current_obj_index is None:
            return
        if 'merged' not in self.obj_type[self.current_obj_index]:
            return
        if blend_mode != self.obj_comp_mode[self.current_obj_index]:
            self.obj_comp_mode[self.current_obj_index] = blend_mode
            self.sig_blend_mode_changed.emit(blend_mode)

    def change_opacity_label_value(self):
        da_val = self.obj_opacity_slider.value()
        self.obj_opacity_val_label.setText('{} %'.format(da_val))

    def send_opacity_changed_signal(self):
        da_val = self.obj_opacity_slider.value()
        if self.current_obj_index is None:
            return
        if 'merged' not in self.obj_type[self.current_obj_index]:
            self.obj_opacity_slider.setValue(da_val)
            return
        self.obj_opacity[self.current_obj_index] = da_val
        self.sig_opacity_changed.emit(da_val)

    def change_size_label_value(self):
        da_val = self.obj_size_slider.value()
        self.obj_size_val_label.setText(str(da_val))

    def send_size_changed_signal(self):
        da_val = self.obj_size_slider.value()
        if self.current_obj_index is None:
            return
        if 'merged' not in self.obj_type[self.current_obj_index]:
            self.obj_size_slider.setValue(da_val)
            return
        self.obj_size[self.current_obj_index] = da_val
        self.sig_size_changed.emit((self.obj_type[self.current_obj_index], da_val))

    def obj_piece_name_changed(self, ev):
        clicked_id = ev[0]
        name = ev[1]
        clicked_index = np.where(np.ravel(self.obj_id) == clicked_id)[0][0]
        self.current_obj_index = clicked_index
        self.obj_name[clicked_index] = name

    def delete_object_btn_clicked(self):
        if self.current_obj_index is None:
            return
        self.delete_objects(self.current_obj_index)

    def obj_clicked(self, clicked_id):
        self.set_active_layer_to_current(clicked_id)
        if 'merged' in self.obj_type[self.current_obj_index]:
            self.obj_info_on_click()

    def obj_info_on_click(self):
        da_data = self.obj_data[self.current_obj_index]
        da_name = self.obj_name[self.current_obj_index]
        if 'probe' in self.obj_type[self.current_obj_index]:
            self.info_window = ProbeInfoWindow(da_name, da_data)
        elif 'virus' in self.obj_type[self.current_obj_index]:
            self.info_window = VirusInfoWindow(da_name, da_data)
        elif 'cell' in self.obj_type[self.current_obj_index]:
            self.info_window = CellsInfoWindow(da_name, da_data)
        else:
            return
        self.info_window.exec()

    def set_active_layer_to_current(self, clicked_id):
        previous_obj_id = self.obj_id[self.current_obj_index]
        if previous_obj_id != clicked_id:
            active_index = np.where(np.ravel(self.obj_id) == clicked_id)[0][0]
            self.current_obj_index = active_index
            # self.obj_list[active_index].set_checked(True)
            inactive_id = np.where(np.ravel(self.obj_id) == previous_obj_id)[0][0]
            self.obj_list[inactive_id].set_checked(False)
            self.set_slider_combo_to_current()
        else:
            return

    def obj_link_changed(self, clicked_id):
        da_index = np.where(np.ravel(self.obj_id) == clicked_id)[0][0]
        if self.obj_list[da_index].link_button.isChecked():
            self.linked_indexes.append(da_index)
        else:
            if da_index in self.linked_indexes:
                del_ind = np.where(np.ravel(self.linked_indexes) == da_index)[0][0]
                self.linked_indexes.pop(del_ind)

    def obj_color_changed(self, clicked_id):
        self.set_active_layer_to_current(clicked_id)
        color = QColorDialog.getColor()
        if color.isValid():
            self.obj_list[self.current_obj_index].set_icon_style(color)
            da_color = (color.red(), color.green(), color.blue(), 255)
            self.sig_color_changed.emit((self.current_obj_index, da_color))

    def obj_eye_clicked(self, ev):
        clicked_id = ev[0]
        vis = ev[1]
        self.set_active_layer_to_current(clicked_id)
        self.sig_visible_changed.emit((self.current_obj_index, vis))

    # delete a object
    def delete_objects(self, delete_index):
        del_ind = np.ravel(delete_index)
        if len(del_ind) > 1:
            del_ind = np.sort(delete_index)[::-1]

        current_obj_id = self.obj_id[self.current_obj_index]
        for da_ind in del_ind:
            self.layer_layout.removeWidget(self.obj_list[da_ind])
            self.obj_list[da_ind].deleteLater()
            del self.obj_list[da_ind]
            del self.obj_id[da_ind]
            del self.obj_name[da_ind]
            del self.obj_type[da_ind]
            del self.obj_data[da_ind]
            del self.obj_comp_mode[da_ind]
            del self.obj_opacity[da_ind]
            del self.obj_size[da_ind]
            self.sig_delete_object.emit(da_ind)
        if self.current_obj_index in del_ind:
            if self.obj_list:
                self.obj_list[-1].set_checked(True)
                self.current_obj_index = len(self.obj_list) - 1
            else:
                self.current_obj_index = None
        else:
            active_index = np.where(np.ravel(self.obj_id) == current_obj_id)[0][0]
            self.current_obj_index = active_index

    #
    def get_object_icon(self, object_type):
        if 'probe' in object_type:
            object_icon = self.probe_icon
        elif 'virus' in object_type:
            object_icon = self.virus_icon
        elif 'cell' in object_type:
            object_icon = self.cell_icon
        elif 'contour' in object_type:
            object_icon = self.contour_icon
        elif 'drawing' in object_type:
            object_icon = self.drawing_icon
        else:
            object_icon = None
        return object_icon

    def get_group_count(self, object_type):
        group_count = len([da_type for da_type in self.obj_type if da_type == object_type])
        return group_count

    def add_object(self, object_name, object_type, object_data, object_mode):
        object_icon = self.get_object_icon(object_type)
        group_count = self.get_group_count(object_type)
        if object_icon is None:
            return
        self.obj_id.append(self.obj_count)
        self.obj_data.append(object_data)
        self.obj_name.append("{} {}".format(object_name, group_count))
        self.obj_type.append(object_type)
        if 'merged' in object_type:
            new_layer = RegisteredObject(obj_id=self.obj_count, obj_name=object_name,
                                         obj_type=object_type, object_icon=object_icon)
            new_layer.eye_clicked.connect(self.obj_eye_clicked)
            new_layer.sig_object_color_changed.connect(self.obj_color_changed)
            new_layer.sig_link.connect(self.obj_link_changed)
            self.obj_opacity.append(self.default_opacity_val)
            self.obj_size.append(self.default_size_val)
            self.obj_comp_mode.append(object_mode)
            da_color = (new_layer.color.red(), new_layer.color.green(), new_layer.color.blue(), 255)
            self.obj_data[-1]['vis_color'] = da_color
            if self.obj_size_slider.value() != self.default_size_val:
                self.obj_size_slider.setValue(self.default_size_val)
            if self.obj_opacity_slider.value() != self.default_opacity_val:
                self.obj_opacity_slider.setValue(self.default_opacity_val)
            if self.obj_blend_combo.currentText() != object_mode:
                self.obj_blend_combo.blockSignals(True)
                self.obj_blend_combo.setCurrentText(object_mode)
                self.obj_blend_combo.blockSignals(False)
        else:
            new_layer = SinglePiece(index=self.obj_count, obj_name=object_name,
                                    obj_type=object_type, object_icon=object_icon)
            new_layer.sig_name_changed.connect(self.obj_piece_name_changed)
            self.obj_opacity.append([])
            self.obj_size.append([])
            self.obj_comp_mode.append([])

        new_layer.text_btn.setText(self.obj_name[-1])
        new_layer.set_checked(True)
        new_layer.sig_clicked.connect(self.obj_clicked)
        self.obj_list.append(new_layer)

        active_index = np.where(np.ravel(self.obj_id) == self.obj_count)[0][0]
        if self.current_obj_index is None:
            self.current_obj_index = active_index
        else:
            self.obj_list[self.current_obj_index].set_checked(False)
            self.current_obj_index = active_index

        self.layer_layout.addWidget(self.obj_list[-1])
        self.obj_count += 1

    # merge object piece
    def get_merged_data(self, obj_type='probe piece'):
        if obj_type not in ['probe piece', 'virus piece', 'contour piece', 'drawing piece', 'cells piece']:
            return
        n_obj = len(self.obj_id)
        cind = [ind for ind in range(n_obj) if obj_type in self.obj_type[ind] and 'merged' not in self.obj_type[ind]]
        da_type_object_names = np.ravel(self.obj_name)[cind]
        n_pieces = len(da_type_object_names)
        pieces_names = []
        for i in range(n_pieces):
            da_name = da_type_object_names[i]
            da_name = da_name.replace(" ", "")
            da_name_split = da_name.split('-')
            pieces_names.append(da_name_split[0])
        unique_object_names = np.unique(pieces_names)
        n_object = len(unique_object_names)
        data = [[] for i in range(n_object)]
        for i in range(n_object):
            da_piece_name = unique_object_names[i]
            da_piece_ind_in_obj_order = [cind[ind] for ind in range(n_pieces) if pieces_names[ind] == da_piece_name]
            temp = self.obj_data[da_piece_ind_in_obj_order[0]].T
            if len(da_piece_ind_in_obj_order) > 1:
                for j in range(1, len(da_piece_ind_in_obj_order)):
                    print(self.obj_data[da_piece_ind_in_obj_order[j]].T)
                    temp = np.hstack([temp, self.obj_data[da_piece_ind_in_obj_order[j]].T])
            data[i] = temp.T
        self.delete_objects(cind)
        return data

    # get obj data
    def get_obj_data(self):
        data = {'obj_name':  self.obj_name,
                'obj_type': self.obj_type,
                'obj_data': self.obj_data,
                'obj_size': self.obj_size,
                'obj_opacity': self.obj_opacity,
                'obj_comp_mode': self.obj_comp_mode,
                'current_obj_index': self.current_obj_index}
        return data

    def set_obj_data(self, data):
        self.obj_name = data['obj_name']
        self.obj_type = data['obj_type']
        self.obj_data = data['obj_data']

        for i in range(len(self.obj_type)):
            self.add_object(self.obj_name[i], self.obj_type[i], self.obj_data[i])

        self.obj_size = data['obj_size']
        self.obj_opacity = data['obj_opacity']
        self.obj_comp_mode = data['obj_comp_mode']
        self.current_obj_index = data['current_obj_index']

        self.obj_list[-1].set_checked(False)
        self.obj_list[self.current_obj_index].set_checked(True)

    def set_slider_combo_to_current(self):
        if 'merged' in self.obj_type[self.current_obj_index]:
            size_val = self.obj_size_slider.value()
            opacity_val = self.obj_opacity_slider.value()
            compo_mode = self.obj_blend_combo.currentText()

            if self.obj_size[self.current_obj_index] != size_val:
                self.obj_size_slider.setValue(self.obj_size[self.current_obj_index])
            if self.obj_opacity[self.current_obj_index] != opacity_val:
                self.obj_opacity_slider.setValue(self.obj_opacity[self.current_obj_index])
            if self.obj_comp_mode[self.current_obj_index] != compo_mode:
                self.obj_blend_combo.setCurrentText(self.obj_comp_mode[self.current_obj_index])
        else:
            return

    def clear_all(self):
        ind = np.arange(len(self.obj_list))[::-1]
        for i in ind:
            self.layer_layout.removeWidget(self.obj_list[i])
            self.obj_list[i].deleteLater()
            del self.obj_list[i]
        self.obj_id = []
        self.obj_name = []
        self.obj_type = []
        self.obj_data = []
        self.obj_size = []
        self.obj_opacity = []
        self.obj_comp_mode = []
        self.obj_count = 0
        self.current_obj_index = []

    def compare_obj_called(self):
        compare_names = [self.obj_name[ind] for ind in self.linked_indexes]
        compare_data = [self.obj_data[ind] for ind in self.linked_indexes]
        info_window = CompareWindow(compare_names, compare_data)
        info_window.exec()





import PyQt5
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *


class PopupMessage(QMessageBox):

    def __init__(self, parent=None):
        QMessageBox.__init__(self)

        self.setWindowTitle("Caution!")
        self.setText('Histological image: is oversized.')
        button = self.exec()
        if button == QMessageBox.Ok:
            print('222')


import cv2
from aicspylibczi import CziFile
from pathlib import Path
from os.path import dirname, realpath, join
import pickle
import numpy as np
import colorsys
from .uuuuuu import hex2rgb

czi_path = '~/Work/Kavli/Data/HERBS_DATA/abraham/Pecorino_mec_slide_1.czi'

class CZIReader(object):
    def __init__(self, czi_path):
        self.is_czi = True
        self.file_name_list = [czi_path[:-4]]
        self.czi = CziFile(czi_path)
        self.czi_info = self.czi.dims
        self.dimensions = self.czi.get_dims_shape()
        self.is_mosaic = self.czi.is_mosaic()
        self.pixel_type = self.czi.pixel_type
        self.n_pages = 1
        self.data = {}
        self.scale = {}

        if 'A' in self.czi_info:
            self.is_rgb = True
            self.n_channels = 3
            if self.pixel_type == 'bgr24':
                self.pixel_type = 'rgb24'
                self.level = 255
                self.data_type = 'uint8'
            else:
                da_power = int(self.pixel_type[-2:]) / 3
                self.pixel_type = 'rgb' + self.pixel_type[-2:]
                self.level = int(np.power(2, da_power)) - 1
                self.data_type = 'uint' + str(int(da_power))
        else:
            self.is_rgb = False
            if self.pixel_type == 'gray16':
                self.level = 65535
                self.data_type = 'uint16'
            else:
                da_power = int(self.pixel_type[-2:]) / 3
                self.level = int(np.power(2, da_power)) - 1
                self.data_type = 'uint' + str(int(da_power))
            self.n_channels = self.dimensions[0]['C'][1]

        self.n_scenes = len(self.dimensions)
        self.scene_bbox = []
        if self.is_mosaic:
            for i in range(self.n_scenes):
                bbox = self.czi.get_mosaic_scene_bounding_box(index=i)
                self.scene_bbox.append((bbox.x, bbox.y, bbox.w, bbox.h))
        else:
            for i in range(self.n_scenes):
                bbox = self.czi.get_scene_bounding_box(index=i)
                self.scene_bbox.append((bbox.x, bbox.y, bbox.w, bbox.h))

        # get colors from metadata
        metadata = self.czi.meta[0]
        all_tags = [metadata[i].tag for i in range(len(metadata))]
        ds_ind = [ind for ind in range(len(all_tags)) if all_tags[ind] == 'DisplaySetting'][0]
        ds_tags = [metadata[ds_ind][i].tag for i in range(len(metadata[ds_ind]))]
        ch_ind = [ind for ind in range(len(ds_tags)) if ds_tags[ind] == 'Channels'][0]
        ch_tags = [metadata[ds_ind][ch_ind][i].tag for i in range(len(metadata[ds_ind][ch_ind]))]

        scale_ind = [ind for ind in range(len(all_tags)) if all_tags[ind] == 'Scaling'][0]
        scale_tags = [metadata[scale_ind][i].tag for i in range(len(metadata[scale_ind]))]
        scale_item_ind = [ind for ind in range(len(scale_tags)) if scale_tags[ind] == 'Items'][0]
        scale_item_tags = [metadata[scale_ind][scale_item_ind][i].tag for i in range(len(metadata[scale_ind][scale_item_ind]))]

        scaling_vals = []
        for i in range(len(scale_item_tags)):
            scale_info = metadata[scale_ind][scale_item_ind]
            single_scaling_tags = [scale_info[i][ind].tag for ind in range(len(scale_info[i]))]
            for j in range(len(single_scaling_tags)):
                scaling_vals.append(metadata[scale_ind][scale_item_ind][i][j].text)

        self.scaling_val = float(scaling_vals[0]) * 1e6

        self.rgb_colors = []
        self.hsv_colors = []
        self.channel_name = []
        self.gamma_val = []

        if self.is_rgb:
            self.rgb_colors = [(255, 0, 0), (0, 255, 0), (0, 0, 255)]
            self.channel_name = ['Red', 'Green', 'Blue']
            for i in range(3):
                chsv = colorsys.rgb_to_hsv(self.rgb_colors[i][0], self.rgb_colors[i][1], self.rgb_colors[i][2])
                hsv_color = (chsv[0], chsv[1], chsv[2] / 255)
                self.hsv_colors.append(hsv_color)
            for i in range(len(ch_tags)):
                chn_info = metadata[ds_ind][ch_ind]
                single_channel_tags = [chn_info[i][ind].tag for ind in range(len(chn_info[i]))]
                channel_vals = []
                for j in range(len(single_channel_tags)):
                    channel_vals.append(metadata[ds_ind][ch_ind][i][j].text)
                if len(np.where(np.ravel(single_channel_tags) == 'Gamma')[0]) > 0:
                    self.gamma_val.append(channel_vals[np.where(np.ravel(single_channel_tags) == 'Gamma')[0][0]])
        else:
            for i in range(len(ch_tags)):
                chn_info = metadata[ds_ind][ch_ind]
                single_channel_tags = [chn_info[i][ind].tag for ind in range(len(chn_info[i]))]
                channel_vals = []
                for j in range(len(single_channel_tags)):
                    channel_vals.append(metadata[ds_ind][ch_ind][i][j].text)
                hex_color = channel_vals[np.where(np.ravel(single_channel_tags) == 'Color')[0][0]]
                self.channel_name.append(channel_vals[np.where(np.ravel(single_channel_tags) == 'ShortName')[0][0]])
                if len(np.where(np.ravel(single_channel_tags) == 'Gamma')[0]) > 0:
                    self.gamma_val.append(channel_vals[np.where(np.ravel(single_channel_tags) == 'Gamma')[0][0]])
                da_color = hex_color[0] + hex_color[3:]
                r, g, b = hex2rgb(da_color)
                chsv = colorsys.rgb_to_hsv(r, g, b)
                hsv_color = (chsv[0], chsv[1], chsv[2] / 255)
                self.hsv_colors.append(hsv_color)
                self.rgb_colors.append((r, g, b))

    def read_data(self, scale, scene_index=None):
        if scene_index is None:
            scene_index = np.arange(self.n_scenes)
        else:
            scene_index = [scene_index]

        for scind in scene_index:
            if self.is_rgb:
                if self.is_mosaic:
                    image_data = self.czi.read_mosaic(C=0, scale_factor=scale, region=self.scene_bbox[scind])
                    if len(image_data.shape) == 4:
                        image_data = image_data[0]
                else:
                    image_data_full = self.czi.read_image(region=self.scene_bbox[scind])
                    image_data = image_data_full[0]
                    image_info = image_data_full[1]
                    if image_info[0][0] == 'C':
                        image_data = image_data[0]
                img = image_data.copy()
                if self.pixel_type == 'rgb24':
                    img_data_temp = img.astype(np.uint8)
                else:
                    img_data_temp = img.astype(np.uint16)
                img_data_temp = cv2.cvtColor(img_data_temp, cv2.COLOR_BGR2RGB)
                self.data['scene %d' % scind] = img_data_temp
                self.scale['scene %d' % scind] = scale
            else:
                if self.n_channels != 1:
                    temp = []
                    for j in range(self.n_channels):
                        mosaic_data = self.czi.read_mosaic(C=j, scale_factor=scale, region=self.scene_bbox[scind])
                        img = mosaic_data[0].copy()
                        temp.append(img)
                    img_data_temp = np.dstack(temp)
                    self.data['scene %d' % scind] = img_data_temp
                    self.scale['scene %d' % scind] = scale
                else:
                    mosaic_data = self.czi.read_mosaic(C=0, scale_factor=scale, region=self.scene_bbox[scind])
                    img = mosaic_data[0].copy()
                    img_data_temp = img
                    self.data['scene %d' % scind] = img_data_temp.reshape(img.shape[0], img.shape[1], 1)
                    self.scale['scene %d' % scind] = scale
